#
# Copyright 2012 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ceilometer.image import glance
from ceilometer.polling import manager
from ceilometer import service
import ceilometer.tests.base as base

IMAGE_LIST = [
    type('Image', (object,),
         {'status': 'active',
          'tags': [],
          'kernel_id': 'fd24d91a-dfd5-4a3c-b990-d4563eb27396',
          'container_format': 'ami',
          'min_ram': 0,
          'ramdisk_id': 'd629522b-ebaa-4c92-9514-9e31fe760d18',
          'updated_at': '2016-06-20T13: 34: 41Z',
          'visibility': 'public',
          'owner': '6824974c08974d4db864bbaa6bc08303',
          'file': '/v2/images/fda54a44-3f96-40bf-ab07-0a4ce9e1761d/file',
          'min_disk': 0,
          'virtual_size': None,
          'id': 'fda54a44-3f96-40bf-ab07-0a4ce9e1761d',
          'size': 25165824,
          'name': 'cirros-0.3.4-x86_64-uec',
          'checksum': 'eb9139e4942121f22bbc2afc0400b2a4',
          'created_at': '2016-06-20T13: 34: 40Z',
          'disk_format': 'ami',
          'is_protected': False,
          'schema': '/v2/schemas/image'}),
    type('Image', (object,),
         {'status': 'active',
          'tags': [],
          'container_format': 'ari',
          'min_ram': 0,
          'updated_at': '2016-06-20T13: 34: 38Z',
          'visibility': 'public',
          'owner': '6824974c08974d4db864bbaa6bc08303',
          'file': '/v2/images/d629522b-ebaa-4c92-9514-9e31fe760d18/file',
          'min_disk': 0,
          'virtual_size': None,
          'id': 'd629522b-ebaa-4c92-9514-9e31fe760d18',
          'size': 3740163,
          'name': 'cirros-0.3.4-x86_64-uec-ramdisk',
          'checksum': 'be575a2b939972276ef675752936977f',
          'created_at': '2016-06-20T13: 34: 37Z',
          'disk_format': 'ari',
          'is_protected': False,
          'schema': '/v2/schemas/image'}),
    type('Image', (object,),
         {'status': 'active',
          'tags': [],
          'container_format': 'aki',
          'min_ram': 0,
          'updated_at': '2016-06-20T13: 34: 35Z',
          'visibility': 'public',
          'owner': '6824974c08974d4db864bbaa6bc08303',
          'file': '/v2/images/fd24d91a-dfd5-4a3c-b990-d4563eb27396/file',
          'min_disk': 0,
          'virtual_size': None,
          'id': 'fd24d91a-dfd5-4a3c-b990-d4563eb27396',
          'size': 4979632,
          'name': 'cirros-0.3.4-x86_64-uec-kernel',
          'checksum': '8a40c862b5735975d82605c1dd395796',
          'created_at': '2016-06-20T13: 34: 35Z',
          'disk_format': 'aki',
          'is_protected': False,
          'schema': '/v2/schemas/image'}),
]


class TestImagePollsterPageSize(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        conf = service.prepare_service([], [])
        self.manager = manager.AgentManager(0, conf)
        self.pollster = glance.ImageSizePollster(conf)

    def test_image_pollster(self):
        image_samples = list(
            self.pollster.get_samples(self.manager, {}, resources=IMAGE_LIST))
        self.assertEqual(3, len(image_samples))
        self.assertEqual('image.size', image_samples[0].name)
        self.assertEqual(25165824, image_samples[0].volume)
        self.assertEqual('6824974c08974d4db864bbaa6bc08303',
                         image_samples[0].project_id)
        self.assertEqual('fda54a44-3f96-40bf-ab07-0a4ce9e1761d',
                         image_samples[0].resource_id)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from ceilometer.polling import plugin_base
from ceilometer import sample

LOG = log.getLogger(__name__)

# Manila share status values mapped to numeric values
SHARE_STATUS = {
    'available': 1,
    'creating': 2,
    'deleting': 3,
    'error': 4,
    'error_deleting': 5,
    'migrating': 6,
    'extending': 7,
    'shrinking': 8,
    'reverting': 9,
    'inactive': 10,
    'manage_starting': 11,
    'manage_error': 12,
    'unmanage_starting': 13,
    'unmanage_error': 14,
    'unmanaged': 15,
    'extending_error': 16,
    'shrinking_error': 17,
    'reverting_error': 18,
    'awaiting_transfer': 19,
    'backup_creating': 20,
    'backup_restoring': 21,
    'backup_restoring_error': 22,
    'creating_from_snapshot': 23,
    'shrinking_possible_data_loss_error': 24,
    'migrating_to': 25,
    'replication_change': 26,
}


class _BaseSharePollster(plugin_base.PollsterBase):
    """Base pollster for Manila share metrics."""

    # Fields to extract from openstacksdk Share objects
    # Note: share_protocol is renamed to 'protocol' in metadata for
    # compatibility with gnocchi_resources.yaml
    FIELDS = ['name',
              'availability_zone',
              'share_protocol',
              'share_type',
              'share_network_id',
              'status',
              'host',
              'is_public',
              ]

    @property
    def default_discovery(self):
        return 'manila_shares'

    @staticmethod
    def extract_metadata(share):
        metadata = {k: getattr(share, k, None)
                    for k in _BaseSharePollster.FIELDS}
        # Rename share_protocol to protocol for gnocchi_resources.yaml
        # compatibility with existing manila_share resource type
        if 'share_protocol' in metadata:
            metadata['protocol'] = metadata.pop('share_protocol')
        return metadata


class ShareStatusPollster(_BaseSharePollster):
    """Pollster for Manila share status."""

    @staticmethod
    def get_status_id(value):
        if not value:
            return -1
        status = value.lower()
        return SHARE_STATUS.get(status, -1)

    def get_samples(self, manager, cache, resources):
        for share in resources or []:
            LOG.debug("Processing share: %s", share.id)
            status = self.get_status_id(share.status)
            if status == -1:
                LOG.warning(
                    "Unknown status %(status)s for share "
                    "%(name)s (%(id)s), setting status to -1",
                    {"status": share.status,
                     "name": share.name,
                     "id": share.id})
            yield sample.Sample(
                name='manila.share.status',
                type=sample.TYPE_GAUGE,
                unit='status',
                volume=status,
                user_id=getattr(share, 'user_id', None),
                project_id=share.project_id,
                resource_id=share.id,
                resource_metadata=self.extract_metadata(share)
            )


class ShareSizePollster(_BaseSharePollster):
    """Pollster for Manila share size."""

    def get_samples(self, manager, cache, resources):
        for share in resources or []:
            LOG.debug("Processing share: %s", share.id)
            size = share.size if share.size is not None else 0
            yield sample.Sample(
                name='manila.share.size',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=size,
                user_id=getattr(share, 'user_id', None),
                project_id=share.project_id,
                resource_id=share.id,
                resource_metadata=self.extract_metadata(share)
            )



#
# Copyright 2013 eNovance
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Publish a sample in memory, useful for testing
"""

from ceilometer import publisher


class TestPublisher(publisher.ConfigPublisherBase):
    """Publisher used in unit testing."""

    def __init__(self, conf, parsed_url):
        super().__init__(conf, parsed_url)
        self.samples = []
        self.events = []
        self.calls = 0

    def publish_samples(self, samples):
        """Send a metering message for publishing

        :param samples: Samples from pipeline after transformation
        """
        self.samples.extend(samples)
        self.calls += 1

    def publish_events(self, events):
        """Send an event message for publishing

        :param events: events from pipeline after transformation
        """
        self.events.extend(events)
        self.calls += 1



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from ceilometer import designate_client
from ceilometer.polling import plugin_base
from ceilometer import sample

LOG = log.getLogger(__name__)

# Designate zone status values mapped to numeric values
ZONE_STATUS = {
    'active': 1,
    'pending': 2,
    'error': 3,
    'deleted': 4,
}

# Designate zone action values mapped to numeric values
ZONE_ACTION = {
    'none': 0,
    'create': 1,
    'delete': 2,
    'update': 3,
}


class _BaseZonePollster(plugin_base.PollsterBase):
    """Base pollster for Designate DNS zone metrics."""

    FIELDS = ['name',
              'email',
              'ttl',
              'description',
              'type',
              'status',
              'action',
              'serial',
              'pool_id',
              ]

    @property
    def default_discovery(self):
        return 'dns_zones'

    @staticmethod
    def extract_metadata(zone):
        return {k: getattr(zone, k, None)
                for k in _BaseZonePollster.FIELDS}


class ZoneStatusPollster(_BaseZonePollster):
    """Pollster for Designate DNS zone status."""

    @staticmethod
    def get_status_id(value):
        if not value:
            return -1
        status = value.lower()
        return ZONE_STATUS.get(status, -1)

    def get_samples(self, manager, cache, resources):
        for zone in resources or []:
            LOG.debug("DNS ZONE: %s", zone)
            status = self.get_status_id(zone.status)
            if status == -1:
                LOG.warning(
                    "Unknown status %(status)s for DNS zone "
                    "%(name)s (%(id)s), setting volume to -1",
                    {"status": zone.status,
                     "name": zone.name,
                     "id": zone.id})
            yield sample.Sample(
                name='dns.zone.status',
                type=sample.TYPE_GAUGE,
                unit='status',
                volume=status,
                user_id=None,
                project_id=zone.project_id,
                resource_id=zone.id,
                resource_metadata=self.extract_metadata(zone)
            )


class ZoneRecordsetCountPollster(_BaseZonePollster):
    """Pollster for Designate DNS zone recordset count."""

    def __init__(self, conf):
        super().__init__(conf)
        self.designate_cli = designate_client.Client(conf)

    def get_samples(self, manager, cache, resources):
        for zone in resources or []:
            LOG.debug("DNS ZONE: %s", zone)
            recordsets = list(self.designate_cli.recordsets_list(zone))
            count = len(recordsets)
            yield sample.Sample(
                name='dns.zone.recordsets',
                type=sample.TYPE_GAUGE,
                unit='recordset',
                volume=count,
                user_id=None,
                project_id=zone.project_id,
                resource_id=zone.id,
                resource_metadata=self.extract_metadata(zone)
            )


class ZoneTTLPollster(_BaseZonePollster):
    """Pollster for Designate DNS zone TTL."""

    def get_samples(self, manager, cache, resources):
        for zone in resources or []:
            LOG.debug("DNS ZONE: %s", zone)
            ttl = zone.ttl if zone.ttl is not None else 0
            yield sample.Sample(
                name='dns.zone.ttl',
                type=sample.TYPE_GAUGE,
                unit='second',
                volume=ttl,
                user_id=None,
                project_id=zone.project_id,
                resource_id=zone.id,
                resource_metadata=self.extract_metadata(zone)
            )


class ZoneSerialPollster(_BaseZonePollster):
    """Pollster for Designate DNS zone serial number."""

    def get_samples(self, manager, cache, resources):
        for zone in resources or []:
            LOG.debug("DNS ZONE: %s", zone)
            serial = zone.serial if zone.serial is not None else 0
            yield sample.Sample(
                name='dns.zone.serial',
                type=sample.TYPE_GAUGE,
                unit='serial',
                volume=serial,
                user_id=None,
                project_id=zone.project_id,
                resource_id=zone.id,
                resource_metadata=self.extract_metadata(zone)
            )



# Copyright 2014 Intel Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Sample data for test_ipmi_sensor.

This data is provided as a sample of the data expected from the ipmitool
binary, which produce Node Manager/IPMI raw data
"""

sensor_temperature_data = """Sensor ID              : SSB Therm Trip (0xd)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Discrete): Temperature
 Assertions Enabled    : Digital State
                         [State Asserted]
 Deassertions Enabled  : Digital State
                         [State Asserted]

Sensor ID              : BB P1 VR Temp (0x20)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Analog)  : Temperature
 Sensor Reading        : 25 (+/- 0) degrees C
 Status                : ok
 Nominal Reading       : 58.000
 Normal Minimum        : 10.000
 Normal Maximum        : 105.000
 Upper critical        : 115.000
 Upper non-critical    : 110.000
 Lower critical        : 0.000
 Lower non-critical    : 5.000
 Positive Hysteresis   : 2.000
 Negative Hysteresis   : 2.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Assertion Events      :
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

Sensor ID              : Front Panel Temp (0x21)
 Entity ID             : 12.1 (Front Panel Board)
 Sensor Type (Analog)  : Temperature
 Sensor Reading        : 23 (+/- 0) degrees C
 Status                : ok
 Nominal Reading       : 28.000
 Normal Minimum        : 10.000
 Normal Maximum        : 45.000
 Upper critical        : 55.000
 Upper non-critical    : 50.000
 Lower critical        : 0.000
 Lower non-critical    : 5.000
 Positive Hysteresis   : 2.000
 Negative Hysteresis   : 2.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Assertion Events      :
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

Sensor ID              : SSB Temp (0x22)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Analog)  : Temperature
 Sensor Reading        : 43 (+/- 0) degrees C
 Status                : ok
 Nominal Reading       : 52.000
 Normal Minimum        : 10.000
 Normal Maximum        : 93.000
 Upper critical        : 103.000
 Upper non-critical    : 98.000
 Lower critical        : 0.000
 Lower non-critical    : 5.000
 Positive Hysteresis   : 2.000
 Negative Hysteresis   : 2.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Assertion Events      :
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

"""

sensor_voltage_data = """Sensor ID              : VR Watchdog (0xb)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Discrete): Voltage
 Assertions Enabled    : Digital State
                         [State Asserted]
 Deassertions Enabled  : Digital State
                         [State Asserted]

Sensor ID              : BB +12.0V (0xd0)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Analog)  : Voltage
 Sensor Reading        : 11.831 (+/- 0) Volts
 Status                : ok
 Nominal Reading       : 11.935
 Normal Minimum        : 11.363
 Normal Maximum        : 12.559
 Upper critical        : 13.391
 Upper non-critical    : 13.027
 Lower critical        : 10.635
 Lower non-critical    : 10.947
 Positive Hysteresis   : 0.052
 Negative Hysteresis   : 0.052
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Assertion Events      :
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

Sensor ID              : BB +1.35 P1LV AB (0xe4)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Analog)  : Voltage
 Sensor Reading        : Disabled
 Status                : Disabled
 Nominal Reading       : 1.342
 Normal Minimum        : 1.275
 Normal Maximum        : 1.409
 Upper critical        : 1.488
 Upper non-critical    : 1.445
 Lower critical        : 1.201
 Lower non-critical    : 1.244
 Positive Hysteresis   : 0.006
 Negative Hysteresis   : 0.006
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Event Status          : Unavailable
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

Sensor ID              : BB +5.0V (0xd1)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Analog)  : Voltage
 Sensor Reading        : 4.959 (+/- 0) Volts
 Status                : ok
 Nominal Reading       : 4.981
 Normal Minimum        : 4.742
 Normal Maximum        : 5.241
 Upper critical        : 5.566
 Upper non-critical    : 5.415
 Lower critical        : 4.416
 Lower non-critical    : 4.546
 Positive Hysteresis   : 0.022
 Negative Hysteresis   : 0.022
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc unc ucr
 Settable Thresholds   : lcr lnc unc ucr
 Threshold Read Mask   : lcr lnc unc ucr
 Assertion Events      :
 Assertions Enabled    : lnc- lcr- unc+ ucr+
 Deassertions Enabled  : lnc- lcr- unc+ ucr+

"""

sensor_current_data = """Sensor ID              : PS1 Curr Out % (0x58)
 Entity ID             : 10.1 (Power Supply)
 Sensor Type (Analog)  : Current
 Sensor Reading        : 11 (+/- 0) unspecified
 Status                : ok
 Nominal Reading       : 50.000
 Normal Minimum        : 0.000
 Normal Maximum        : 100.000
 Upper critical        : 118.000
 Upper non-critical    : 100.000
 Positive Hysteresis   : Unspecified
 Negative Hysteresis   : Unspecified
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : unc ucr
 Settable Thresholds   : unc ucr
 Threshold Read Mask   : unc ucr
 Assertion Events      :
 Assertions Enabled    : unc+ ucr+
 Deassertions Enabled  : unc+ ucr+

Sensor ID              : PS2 Curr Out % (0x59)
 Entity ID             : 10.2 (Power Supply)
 Sensor Type (Analog)  : Current
 Sensor Reading        : 0 (+/- 0) unspecified
 Status                : ok
 Nominal Reading       : 50.000
 Normal Minimum        : 0.000
 Normal Maximum        : 100.000
 Upper critical        : 118.000
 Upper non-critical    : 100.000
 Positive Hysteresis   : Unspecified
 Negative Hysteresis   : Unspecified
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : unc ucr
 Settable Thresholds   : unc ucr
 Threshold Read Mask   : unc ucr
 Assertion Events      :
 Assertions Enabled    : unc+ ucr+
 Deassertions Enabled  : unc+ ucr+

Sensor ID              : Pwr Consumption (0x76)
 Entity ID             : 7.1 (System Board)
 Sensor Type (Threshold)  : Current (0x03)
 Sensor Reading        : 154 (+/- 0) Watts
 Status                : ok
 Nominal Reading       : 1034.000
 Normal Maximum        : 1056.000
 Upper critical        : 1914.000
 Upper non-critical    : 1738.000
 Positive Hysteresis   : Unspecified
 Negative Hysteresis   : Unspecified
 Minimum sensor range  : Unspecified
 Maximum sensor range  : 5588.000
 Event Message Control : Per-threshold
 Readable Thresholds   : unc ucr
 Settable Thresholds   : unc
 Assertion Events      :
 Assertions Enabled    : unc+ ucr+
 Deassertions Enabled  : unc+ ucr+

"""

sensor_fan_data = """Sensor ID              : System Fan 1 (0x30)
 Entity ID             : 29.1 (Fan Device)
 Sensor Type (Analog)  : Fan
 Sensor Reading        : 4704 (+/- 0) RPM
 Status                : ok
 Nominal Reading       : 7497.000
 Normal Minimum        : 2499.000
 Normal Maximum        : 12495.000
 Lower critical        : 1715.000
 Lower non-critical    : 1960.000
 Positive Hysteresis   : 49.000
 Negative Hysteresis   : 49.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc
 Settable Thresholds   : lcr lnc
 Threshold Read Mask   : lcr lnc
 Assertion Events      :
 Assertions Enabled    : lnc- lcr-
 Deassertions Enabled  : lnc- lcr-

Sensor ID              : System Fan 2 (0x32)
 Entity ID             : 29.2 (Fan Device)
 Sensor Type (Analog)  : Fan
 Sensor Reading        : 4704 (+/- 0) RPM
 Status                : ok
 Nominal Reading       : 7497.000
 Normal Minimum        : 2499.000
 Normal Maximum        : 12495.000
 Lower critical        : 1715.000
 Lower non-critical    : 1960.000
 Positive Hysteresis   : 49.000
 Negative Hysteresis   : 49.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc
 Settable Thresholds   : lcr lnc
 Threshold Read Mask   : lcr lnc
 Assertion Events      :
 Assertions Enabled    : lnc- lcr-
 Deassertions Enabled  : lnc- lcr-

Sensor ID              : System Fan 3 (0x34)
 Entity ID             : 29.3 (Fan Device)
 Sensor Type (Analog)  : Fan
 Sensor Reading        : 4704 (+/- 0) RPM
 Status                : ok
 Nominal Reading       : 7497.000
 Normal Minimum        : 2499.000
 Normal Maximum        : 12495.000
 Lower critical        : 1715.000
 Lower non-critical    : 1960.000
 Positive Hysteresis   : 49.000
 Negative Hysteresis   : 49.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc
 Settable Thresholds   : lcr lnc
 Threshold Read Mask   : lcr lnc
 Assertion Events      :
 Assertions Enabled    : lnc- lcr-
 Deassertions Enabled  : lnc- lcr-

Sensor ID              : System Fan 4 (0x36)
 Entity ID             : 29.4 (Fan Device)
 Sensor Type (Analog)  : Fan
 Sensor Reading        : 4606 (+/- 0) RPM
 Status                : ok
 Nominal Reading       : 7497.000
 Normal Minimum        : 2499.000
 Normal Maximum        : 12495.000
 Lower critical        : 1715.000
 Lower non-critical    : 1960.000
 Positive Hysteresis   : 49.000
 Negative Hysteresis   : 49.000
 Minimum sensor range  : Unspecified
 Maximum sensor range  : Unspecified
 Event Message Control : Per-threshold
 Readable Thresholds   : lcr lnc
 Settable Thresholds   : lcr lnc
 Threshold Read Mask   : lcr lnc
 Assertion Events      :
 Assertions Enabled    : lnc- lcr-
 Deassertions Enabled  : lnc- lcr-

"""


sensor_status_cmd = 'ipmitoolraw0x0a0x2c0x00'
init_sensor_cmd = 'ipmitoolraw0x0a0x2c0x01'
sdr_info_cmd = 'ipmitoolsdrinfo'

read_sensor_all_cmd = 'ipmitoolsdr-v'
read_sensor_temperature_cmd = 'ipmitoolsdr-vtypeTemperature'
read_sensor_voltage_cmd = 'ipmitoolsdr-vtypeVoltage'
read_sensor_current_cmd = 'ipmitoolsdr-vtypeCurrent'
read_sensor_fan_cmd = 'ipmitoolsdr-vtypeFan'

sdr_info = ('', '')

sensor_temperature = (sensor_temperature_data, '')
sensor_voltage = (sensor_voltage_data, '')
sensor_current = (sensor_current_data, '')
sensor_fan = (sensor_fan_data, '')



# Copyright (C) 2015 Reliance Jio Infocomm Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from oslotest import base

from ceilometer.objectstore import rgw_client


RGW_ADMIN_BUCKETS = '''
[
  {
    "max_marker": "",
    "ver": 2001,
    "usage": {
      "rgw.main": {
        "size_kb_actual": 16000,
        "num_objects": 1000,
        "size_kb": 1000
      }
    },
    "bucket": "somefoo",
    "owner": "admin",
    "master_ver": 0,
    "mtime": 1420176126,
    "marker": "default.4126.1",
    "bucket_quota": {
      "max_objects": -1,
      "enabled": false,
      "max_size_kb": -1
    },
    "id": "default.4126.1",
    "pool": ".rgw.buckets",
    "index_pool": ".rgw.buckets.index"
  },
  {
    "max_marker": "",
    "ver": 3,
    "usage": {
      "rgw.main": {
        "size_kb_actual": 43,
        "num_objects": 1,
        "size_kb": 42
      }
    },
    "bucket": "somefoo31",
    "owner": "admin",
    "master_ver": 0,
    "mtime": 1420176134,
    "marker": "default.4126.5",
    "bucket_quota": {
      "max_objects": -1,
      "enabled": false,
      "max_size_kb": -1
    },
    "id": "default.4126.5",
    "pool": ".rgw.buckets",
    "index_pool": ".rgw.buckets.index"
  }
]'''

RGW_ADMIN_USAGE = '''
{ "entries": [
        { "owner": "5f7fe2d5352e466f948f49341e33d107",
          "buckets": [
                { "bucket": "",
                  "time": "2015-01-23 09:00:00.000000Z",
                  "epoch": 1422003600,
                  "categories": [
                        { "category": "list_buckets",
                          "bytes_sent": 46,
                          "bytes_received": 0,
                          "ops": 3,
                          "successful_ops": 3},
                        { "category": "stat_account",
                          "bytes_sent": 0,
                          "bytes_received": 0,
                          "ops": 1,
                          "successful_ops": 1}]},
                { "bucket": "foodsgh",
                  "time": "2015-01-23 09:00:00.000000Z",
                  "epoch": 1422003600,
                  "categories": [
                        { "category": "create_bucket",
                          "bytes_sent": 0,
                          "bytes_received": 0,
                          "ops": 1,
                          "successful_ops": 1},
                        { "category": "get_obj",
                          "bytes_sent": 0,
                          "bytes_received": 0,
                          "ops": 1,
                          "successful_ops": 0},
                        { "category": "put_obj",
                          "bytes_sent": 0,
                          "bytes_received": 238,
                          "ops": 1,
                          "successful_ops": 1}]}]}],
  "summary": [
        { "user": "5f7fe2d5352e466f948f49341e33d107",
          "categories": [
                { "category": "create_bucket",
                  "bytes_sent": 0,
                  "bytes_received": 0,
                  "ops": 1,
                  "successful_ops": 1},
                { "category": "get_obj",
                  "bytes_sent": 0,
                  "bytes_received": 0,
                  "ops": 1,
                  "successful_ops": 0},
                { "category": "list_buckets",
                  "bytes_sent": 46,
                  "bytes_received": 0,
                  "ops": 3,
                  "successful_ops": 3},
                { "category": "put_obj",
                  "bytes_sent": 0,
                  "bytes_received": 238,
                  "ops": 1,
                  "successful_ops": 1},
                { "category": "stat_account",
                  "bytes_sent": 0,
                  "bytes_received": 0,
                  "ops": 1,
                  "successful_ops": 1}],
          "total": { "bytes_sent": 46,
              "bytes_received": 238,
              "ops": 7,
              "successful_ops": 6}}]}
'''

buckets_json = json.loads(RGW_ADMIN_BUCKETS)
usage_json = json.loads(RGW_ADMIN_USAGE)


class TestRGWAdminClient(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.client = rgw_client.RGWAdminClient('http://127.0.0.1:8080/admin',
                                                'abcde', 'secret', False)
        self.get_resp = mock.MagicMock()
        self.get = mock.patch('requests.request',
                              return_value=self.get_resp).start()

    def test_make_request_exception(self):
        self.get_resp.status_code = 403
        self.assertRaises(rgw_client.RGWAdminAPIFailed,
                          self.client._make_request,
                          *('foo', {}))

    def test_make_request(self):
        self.get_resp.status_code = 200
        self.get_resp.json.return_value = buckets_json
        actual = self.client._make_request('foo', [])
        self.assertEqual(buckets_json, actual)

    def test_get_buckets(self):
        self.get_resp.status_code = 200
        self.get_resp.json.return_value = buckets_json
        actual = self.client.get_bucket('foo')
        bucket_list = [rgw_client.RGWAdminClient.Bucket('somefoo', 1000, 1000),
                       rgw_client.RGWAdminClient.Bucket('somefoo31', 1, 42),
                       ]
        expected = {'num_buckets': 2, 'size': 1042, 'num_objects': 1001,
                    'buckets': bucket_list}
        self.assertEqual(expected, actual)
        self.assertEqual(1, len(self.get.call_args_list))
        self.assertEqual('http://127.0.0.1:8080/admin/bucket?'
                         'uid=foo&stats=true',
                         self.get.call_args_list[0][0][1])

    def test_get_buckets_implicit_tenants(self):
        self.get_resp.status_code = 200
        self.get_resp.json.return_value = buckets_json
        self.client.implicit_tenants = True
        actual = self.client.get_bucket('foo')
        bucket_list = [rgw_client.RGWAdminClient.Bucket('somefoo', 1000, 1000),
                       rgw_client.RGWAdminClient.Bucket('somefoo31', 1, 42),
                       ]
        expected = {'num_buckets': 2, 'size': 1042, 'num_objects': 1001,
                    'buckets': bucket_list}
        self.assertEqual(expected, actual)
        self.assertEqual(1, len(self.get.call_args_list))
        self.assertEqual('http://127.0.0.1:8080/admin/bucket?'
                         'uid=foo$foo&stats=true',
                         self.get.call_args_list[0][0][1])

    def test_get_usage(self):
        self.get_resp.status_code = 200
        self.get_resp.json.return_value = usage_json
        actual = self.client.get_usage('foo')
        expected = 7
        self.assertEqual(expected, actual)
        self.assertEqual(1, len(self.get.call_args_list))
        self.assertEqual('http://127.0.0.1:8080/admin/usage?uid=foo',
                         self.get.call_args_list[0][0][1])

    def test_get_usage_implicit_tenants(self):
        self.get_resp.status_code = 200
        self.get_resp.json.return_value = usage_json
        self.client.implicit_tenants = True
        actual = self.client.get_usage('foo')
        expected = 7
        self.assertEqual(expected, actual)
        self.assertEqual(1, len(self.get.call_args_list))
        self.assertEqual('http://127.0.0.1:8080/admin/usage?uid=foo$foo',
                         self.get.call_args_list[0][0][1])



#
# Copyright 2012 eNovance <licensing@enovance.com>
# Copyright 2012 Red Hat, Inc
# Copyright 2014 Cisco Systems, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from ceilometer.compute import pollsters
from ceilometer import sample


class PerDeviceDiskPollster(pollsters.GenericComputePollster):
    inspector_method = "inspect_disks"

    @staticmethod
    def get_resource_id(instance, stats):
        return f"{instance.id}-{stats.device}"

    @staticmethod
    def get_additional_metadata(instance, stats):
        return {'disk_name': stats.device}


class PerDeviceReadRequestsPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.read.requests'
    sample_unit = 'request'
    sample_type = sample.TYPE_CUMULATIVE
    sample_stats_key = 'read_requests'


class PerDeviceReadBytesPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.read.bytes'
    sample_unit = 'B'
    sample_type = sample.TYPE_CUMULATIVE
    sample_stats_key = 'read_bytes'


class PerDeviceWriteRequestsPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.write.requests'
    sample_unit = 'request'
    sample_type = sample.TYPE_CUMULATIVE
    sample_stats_key = 'write_requests'


class PerDeviceWriteBytesPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.write.bytes'
    sample_unit = 'B'
    sample_type = sample.TYPE_CUMULATIVE
    sample_stats_key = 'write_bytes'


class PerDeviceCapacityPollster(PerDeviceDiskPollster):
    inspector_method = 'inspect_disk_info'
    sample_name = 'disk.device.capacity'
    sample_unit = 'B'
    sample_stats_key = 'capacity'


class PerDeviceAllocationPollster(PerDeviceDiskPollster):
    inspector_method = 'inspect_disk_info'
    sample_name = 'disk.device.allocation'
    sample_unit = 'B'
    sample_stats_key = 'allocation'


class PerDevicePhysicalPollster(PerDeviceDiskPollster):
    inspector_method = 'inspect_disk_info'
    sample_name = 'disk.device.usage'
    sample_unit = 'B'
    sample_stats_key = 'physical'


class PerDeviceDiskReadLatencyPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.read.latency'
    sample_type = sample.TYPE_CUMULATIVE
    sample_unit = 'ns'
    sample_stats_key = 'rd_total_times'


class PerDeviceDiskWriteLatencyPollster(PerDeviceDiskPollster):
    sample_name = 'disk.device.write.latency'
    sample_type = sample.TYPE_CUMULATIVE
    sample_unit = 'ns'
    sample_stats_key = 'wr_total_times'


class EphemeralSizePollster(pollsters.InstanceMetadataPollster):
    sample_name = 'disk.ephemeral.size'
    sample_unit = 'GiB'

    def get_volume(self, instance):
        return int(instance.flavor['ephemeral'])


class RootSizePollster(pollsters.InstanceMetadataPollster):
    sample_name = 'disk.root.size'
    sample_unit = 'GiB'

    def get_volume(self, instance):
        return (int(instance.flavor['disk'])
                - int(instance.flavor['ephemeral']))



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Common code for working with alarm metrics
"""
from ceilometer.polling import plugin_base
from ceilometer import sample

DEFAULT_GROUP = "service_credentials"


class _Base(plugin_base.PollsterBase):
    @property
    def default_discovery(self):
        return 'alarm'


class EvaluationResultPollster(_Base):
    @staticmethod
    def get_evaluation_results_metrics(metrics):
        evaluation_metrics = []
        if "evaluation_results" in metrics:
            for metric in metrics["evaluation_results"]:
                for state, count in metric["state_counters"].items():
                    evaluation_metrics.append({
                        "name": "evaluation_result",
                        "state": state,
                        "count": count,
                        "project_id": metric['project_id'],
                        "alarm_id": metric['alarm_id']
                    })
        return evaluation_metrics

    def get_samples(self, manager, cache, resources):
        metrics = self.get_evaluation_results_metrics(resources[0])
        for metric in metrics:
            yield sample.Sample(
                name='alarm.' + metric['name'],
                type=sample.TYPE_GAUGE,
                volume=int(metric['count']),
                unit='evaluation_result_count',
                user_id=None,
                project_id=metric['project_id'],
                resource_id=metric['alarm_id'],
                resource_metadata={"alarm_state": metric['state']},
            )



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from gnocchiclient import client
from gnocchiclient import exceptions as gnocchi_exc
import keystoneauth1.session
from oslo_log import log
from oslo_utils import versionutils

from ceilometer import keystone_client

LOG = log.getLogger(__name__)


def get_gnocchiclient(conf, request_timeout=None):
    group = conf.gnocchi.auth_section
    session = keystone_client.get_session(conf, group=group,
                                          timeout=request_timeout)
    adapter = keystoneauth1.session.TCPKeepAliveAdapter(
        pool_maxsize=conf.max_parallel_requests)
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    interface = conf[group].interface
    region_name = conf[group].region_name
    gnocchi_url = session.get_endpoint(service_type='metric',
                                       service_name='gnocchi',
                                       interface=interface,
                                       region_name=region_name)
    return client.Client(
        '1', session, adapter_options={'connect_retries': 3,
                                       'interface': interface,
                                       'region_name': region_name,
                                       'endpoint_override': gnocchi_url})


# NOTE(sileht): This is the initial resource types created in Gnocchi
# This list must never change to keep in sync with what Gnocchi early
# database contents was containing
resources_initial = {
    "image": {
        "name": {"type": "string", "min_length": 0, "max_length": 255,
                 "required": True},
        "container_format": {"type": "string", "min_length": 0,
                             "max_length": 255, "required": True},
        "disk_format": {"type": "string", "min_length": 0, "max_length": 255,
                        "required": True},
    },
    "instance": {
        "flavor_id": {"type": "string", "min_length": 0, "max_length": 255,
                      "required": True},
        "image_ref": {"type": "string", "min_length": 0, "max_length": 255,
                      "required": False},
        "host": {"type": "string", "min_length": 0, "max_length": 255,
                 "required": True},
        "display_name": {"type": "string", "min_length": 0, "max_length": 255,
                         "required": True},
        "server_group": {"type": "string", "min_length": 0, "max_length": 255,
                         "required": False},
    },
    "instance_disk": {
        "name": {"type": "string", "min_length": 0, "max_length": 255,
                 "required": True},
        "instance_id": {"type": "uuid", "required": True},
    },
    "instance_network_interface": {
        "name": {"type": "string", "min_length": 0, "max_length": 255,
                 "required": True},
        "instance_id": {"type": "uuid", "required": True},
    },
    "volume": {
        "display_name": {"type": "string", "min_length": 0, "max_length": 255,
                         "required": False},
    },
    "swift_account": {},
    "ceph_account": {},
    "network": {},
    "identity": {},
    "ipmi": {},
    "stack": {},
    "host": {
        "host_name": {"type": "string", "min_length": 0, "max_length": 255,
                      "required": True},
    },
    "host_network_interface": {
        "host_name": {"type": "string", "min_length": 0, "max_length": 255,
                      "required": True},
        "device_name": {"type": "string", "min_length": 0, "max_length": 255,
                        "required": False},
    },
    "host_disk": {
        "host_name": {"type": "string", "min_length": 0, "max_length": 255,
                      "required": True},
        "device_name": {"type": "string", "min_length": 0, "max_length": 255,
                        "required": False},
    },
}

# NOTE(sileht): Order matter this have to be considered like alembic migration
# code, because it updates the resources schema of Gnocchi
resources_update_operations = [
    {"desc": "add volume_type to volume",
     "type": "update_attribute_type",
     "resource_type": "volume",
     "data": [{
         "op": "add",
         "path": "/attributes/volume_type",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}
     }]},
    {"desc": "add flavor_name to instance",
     "type": "update_attribute_type",
     "resource_type": "instance",
     "data": [{
         "op": "add",
         "path": "/attributes/flavor_name",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": True, "options": {'fill': ''}}
     }]},
    {"desc": "add nova_compute resource type",
     "type": "create_resource_type",
     "resource_type": "nova_compute",
     "data": [{
         "attributes": {"host_name": {"type": "string", "min_length": 0,
                        "max_length": 255, "required": True}}
     }]},
    {"desc": "add manila share type",
     "type": "create_resource_type",
     "resource_type": "manila_share",
     "data": [{
         "attributes": {"name": {"type": "string", "min_length": 0,
                                 "max_length": 255, "required": False},
                        "host": {"type": "string", "min_length": 0,
                                 "max_length": 255, "required": True},
                        "protocol": {"type": "string", "min_length": 0,
                                     "max_length": 255, "required": False},
                        "availability_zone": {"type": "string",
                                              "min_length": 0,
                                              "max_length": 255,
                                              "required": False},
                        "status": {"type": "string", "min_length": 0,
                                   "max_length": 255,
                                   "required": True}}
     }]},
    {"desc": "add volume provider resource type",
     "type": "create_resource_type",
     "resource_type": "volume_provider",
     "data": [{
         "attributes": {}
     }]},
    {"desc": "add volume provider pool resource type",
     "type": "create_resource_type",
     "resource_type": "volume_provider_pool",
     "data": [{
         "attributes": {"provider": {"type": "string", "min_length": 0,
                                     "max_length": 255, "required": True}}
     }]},
    {"desc": "add ipmi sensor resource type",
     "type": "create_resource_type",
     "resource_type": "ipmi_sensor",
     "data": [{
         "attributes": {"node": {"type": "string", "min_length": 0,
                                 "max_length": 255, "required": True}}
     }]},
    {"desc": "add launched_at to instance",
     "type": "update_attribute_type",
     "resource_type": "instance",
     "data": [
         {"op": "add", "path": "/attributes/launched_at",
          "value": {"type": "datetime", "required": False}},
         {"op": "add", "path": "/attributes/created_at",
          "value": {"type": "datetime", "required": False}},
         {"op": "add", "path": "/attributes/deleted_at",
          "value": {"type": "datetime", "required": False}},
     ]},
    {"desc": "add instance_id/image_id to volume",
     "type": "update_attribute_type",
     "resource_type": "volume",
     "data": [
         {"op": "add", "path": "/attributes/image_id",
          "value": {"type": "uuid", "required": False}},
         {"op": "add", "path": "/attributes/instance_id",
          "value": {"type": "uuid", "required": False}},
     ]},
    {"desc": "add availability_zone to instance",
     "type": "update_attribute_type",
     "resource_type": "instance",
     "data": [{
         "op": "add",
         "path": "/attributes/availability_zone",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}
     }]},
    {"desc": "add volume_type_id to volume",
     "type": "update_attribute_type",
     "resource_type": "volume",
     "data": [{
         "op": "add",
         "path": "/attributes/volume_type_id",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}
     }]},
    {"desc": "add storage_policy to swift_account",
     "type": "update_attribute_type",
     "resource_type": "swift_account",
     "data": [{
         "op": "add",
         "path": "/attributes/storage_policy",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}  # Only containers have a storage policy
     }]},
    {"desc": "make host optional for instance",
     "type": "update_attribute_type",
     "resource_type": "instance",
     "data": [{
         "op": "add",  # Usually update, the attribute likely already exists
         "path": "/attributes/host",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}  # Allow the hypervisor to be withheld
     }]},
    {"desc": "make container_format optional for image",
     "type": "update_attribute_type",
     "resource_type": "image",
     "data": [{
         "op": "add",  # Usually update, the attribute likely already exists
         "path": "/attributes/container_format",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}  # This can be null in certain cases
     }]},
    {"desc": "make disk_format optional for image",
     "type": "update_attribute_type",
     "resource_type": "image",
     "data": [{
         "op": "add",  # Usually update, the attribute likely already exists
         "path": "/attributes/disk_format",
         "value": {"type": "string", "min_length": 0, "max_length": 255,
                   "required": False}  # This can be null in certain cases
     }]},
    {"desc": "add loadbalancer resource type",
     "type": "create_resource_type",
     "resource_type": "loadbalancer",
     "data": [{
         "attributes": {
             "name": {"type": "string", "min_length": 0,
                      "max_length": 255, "required": False},
             "availability_zone": {"type": "string", "min_length": 0,
                                   "max_length": 255, "required": False},
             "vip_address": {"type": "string", "min_length": 0,
                             "max_length": 255, "required": False},
             "provider": {"type": "string", "min_length": 0,
                          "max_length": 255, "required": False}
         }
     }]},
    {"desc": "add dns_zone resource type",
     "type": "create_resource_type",
     "resource_type": "dns_zone",
     "data": [{
         "attributes": {
             "zone_name": {"type": "string", "min_length": 0,
                           "max_length": 255, "required": False},
             "email": {"type": "string", "min_length": 0,
                       "max_length": 255, "required": False},
             "zone_type": {"type": "string", "min_length": 0,
                           "max_length": 255, "required": False},
             "pool_id": {"type": "string", "min_length": 0,
                         "max_length": 255, "required": False}
         }
     }]},
]


REQUIRED_VERSION = "4.2.0"


def upgrade_resource_types(conf):
    gnocchi = get_gnocchiclient(conf)

    gnocchi_version = gnocchi.build.get()
    if not versionutils.is_compatible(REQUIRED_VERSION, gnocchi_version):
        raise Exception("required gnocchi version is %s, got %s" %
                        (REQUIRED_VERSION, gnocchi_version))

    for name, attributes in resources_initial.items():
        try:
            gnocchi.resource_type.get(name=name)
        except (gnocchi_exc.ResourceTypeNotFound, gnocchi_exc.NotFound):
            rt = {'name': name, 'attributes': attributes}
            gnocchi.resource_type.create(resource_type=rt)

    for ops in resources_update_operations:
        if ops['type'] == 'update_attribute_type':
            rt = gnocchi.resource_type.get(name=ops['resource_type'])
            first_op = ops['data'][0]
            attrib = first_op['path'].replace('/attributes/', '')
            # Options are only used when adding/updating attributes.
            # Make a shallow copy of the new value type, and remove options
            # from the copy to make sure it isn't included in checks.
            value = first_op['value'].copy()
            value.pop('options', None)
            if (first_op['op'] == 'add'
                    and attrib in rt['attributes']
                    and value == rt['attributes'][attrib]):
                continue
            if first_op['op'] == 'remove' and attrib not in rt['attributes']:
                continue
            gnocchi.resource_type.update(ops['resource_type'], ops['data'])
        elif ops['type'] == 'create_resource_type':
            try:
                gnocchi.resource_type.get(name=ops['resource_type'])
            except (gnocchi_exc.ResourceTypeNotFound, gnocchi_exc.NotFound):
                rt = {'name': ops['resource_type'],
                      'attributes': ops['data'][0]['attributes']}
                gnocchi.resource_type.create(resource_type=rt)



# Copyright (C) 2014 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools

from openstack import connection
from openstack import exceptions as os_exc
from oslo_config import cfg
from oslo_log import log

from ceilometer import keystone_client

SERVICE_OPTS = [
    cfg.StrOpt('neutron',
               default='network',
               help='Neutron service type.'),
]

LOG = log.getLogger(__name__)


def logged(func):
    """Decorator to log exceptions from openstacksdk calls."""

    @functools.wraps(func)
    def with_logging(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except os_exc.HttpException as e:
            if e.status_code == 404:
                LOG.warning("The resource could not be found.")
            else:
                LOG.warning(e)
            return []
        except Exception as e:
            LOG.exception(e)
            raise

    return with_logging


class Client:
    """A client which gets information via openstacksdk."""

    def __init__(self, conf):
        """Initialize the Neutron client using openstacksdk.

        :param conf: Oslo config object with service credentials.
        """
        creds = conf.service_credentials
        params = {
            'session': keystone_client.get_session(conf),
            'endpoint_type': creds.interface,
            'region_name': creds.region_name,
            'service_type': conf.service_types.neutron,
        }
        self.conn = connection.Connection(conf=conf, **params)

    @logged
    def vpn_get_all(self):
        """Get all VPN services."""
        return [vpn.to_dict() for vpn in self.conn.network.vpn_services()]

    @logged
    def ipsec_site_connections_get_all(self):
        """Get all IPSec site connections."""
        return [conn.to_dict()
                for conn in self.conn.network.vpn_ipsec_site_connections()]

    @logged
    def firewall_get_all(self):
        """Get all firewall groups (FWaaS v2)."""
        return [fw.to_dict()
                for fw in self.conn.network.firewall_groups()]

    @logged
    def fw_policy_get_all(self):
        """Get all firewall policies."""
        return [policy.to_dict()
                for policy in self.conn.network.firewall_policies()]

    @logged
    def fip_get_all(self):
        """Get all floating IPs."""
        return [fip.to_dict() for fip in self.conn.network.ips()]



# Copyright 2012 New Dream Network (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test base classes.
"""
import functools
import os
import tempfile
import unittest

import fixtures
import oslo_messaging.conffixture
from oslotest import base
import yaml

import ceilometer
from ceilometer import messaging


class BaseTestCase(base.BaseTestCase):
    def setup_messaging(self, conf, exchange=None):
        self.useFixture(oslo_messaging.conffixture.ConfFixture(conf))
        conf.set_override("notification_driver", ["messaging"])
        if not exchange:
            exchange = 'ceilometer'
        conf.set_override("control_exchange", exchange)

        # NOTE(sileht): Ensure a new oslo.messaging driver is loaded
        # between each tests
        self.transport = messaging.get_transport(conf, "fake://", cache=False)
        self.useFixture(fixtures.MockPatch(
            'ceilometer.messaging.get_transport',
            return_value=self.transport))

    def cfg2file(self, data):
        cfgfile = tempfile.NamedTemporaryFile(mode='w', delete=False)
        self.addCleanup(os.remove, cfgfile.name)
        cfgfile.write(yaml.safe_dump(data))
        cfgfile.close()
        return cfgfile.name

    @staticmethod
    def path_get(project_file=None):
        root = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                            '..',
                                            '..',
                                            )
                               )
        if project_file:
            return os.path.join(root, project_file)
        else:
            return root


def _skip_decorator(func):
    @functools.wraps(func)
    def skip_if_not_implemented(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ceilometer.NotImplementedError as e:
            raise unittest.SkipTest(str(e))
    return skip_if_not_implemented


class SkipNotImplementedMeta(type):
    def __new__(cls, name, bases, local):
        for attr in local:
            value = local[attr]
            if callable(value) and (
                    attr.startswith('test_') or attr == 'setUp'):
                local[attr] = _skip_decorator(value)
        return type.__new__(cls, name, bases, local)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Dynamic pollster component
    This component enables operators to create new pollsters on the fly
    via configuration. The configuration files are read from
    '/etc/ceilometer/pollsters.d/'. The pollster are defined in YAML files
    similar to the idea used for handling notifications.
"""
import copy
import json
import re
import subprocess
import time
import xmltodict

from oslo_log import log

from requests import RequestException

from ceilometer import declarative
from ceilometer.polling import plugin_base
from ceilometer import sample as ceilometer_sample
from ceilometer import utils as ceilometer_utils

from functools import reduce
import operator
import requests

from urllib import parse as urlparse

LOG = log.getLogger(__name__)


def validate_sample_type(sample_type):
    if sample_type not in ceilometer_sample.TYPES:
        raise declarative.DynamicPollsterDefinitionException(
            "Invalid sample type [%s]. Valid ones are [%s]."
            % (sample_type, ceilometer_sample.TYPES))


class XMLResponseHandler:
    """This response handler converts an XML in string format to a dict"""

    @staticmethod
    def handle(response):
        return xmltodict.parse(response)


class JsonResponseHandler:
    """This response handler converts a JSON in string format to a dict"""

    @staticmethod
    def handle(response):
        return json.loads(response)


class PlainTextResponseHandler:
    """Response handler converts string to a list of dict [{'out'=<string>}]"""

    @staticmethod
    def handle(response):
        return [{'out': str(response)}]


VALID_HANDLERS = {
    'json': JsonResponseHandler,
    'xml': XMLResponseHandler,
    'text': PlainTextResponseHandler
}


def validate_response_handler(val):
    if not isinstance(val, list):
        raise declarative.DynamicPollsterDefinitionException(
            "Invalid response_handlers configuration. It must be a list. "
            "Provided value type: %s" % type(val).__name__)

    for value in val:
        if value not in VALID_HANDLERS:
            raise declarative.DynamicPollsterDefinitionException(
                "Invalid response_handler value [%s]. Accepted values "
                "are [%s]" % (value, ', '.join(list(VALID_HANDLERS))))


def validate_extra_metadata_skip_samples(val):
    if not isinstance(val, list) or next(
            filter(lambda v: not isinstance(v, dict), val), None):
        raise declarative.DynamicPollsterDefinitionException(
            "Invalid extra_metadata_fields_skip configuration."
            " It must be a list of maps. Provided value: %s,"
            " value type: %s." % (val, type(val).__name__))


class ResponseHandlerChain:
    """Tries to convert a string to a dict using the response handlers"""

    def __init__(self, response_handlers, **meta):
        if not isinstance(response_handlers, list):
            response_handlers = list(response_handlers)

        self.response_handlers = response_handlers
        self.meta = meta

    def handle(self, response):
        failed_handlers = []
        for handler in self.response_handlers:
            try:
                return handler.handle(response)
            except Exception as e:
                handler_name = handler.__name__
                failed_handlers.append(handler_name)
                LOG.debug(
                    "Error handling response [%s] with handler [%s]: %s. "
                    "We will try the next one, if multiple handlers were "
                    "configured.",
                    response, handler_name, e)

        handlers_str = ', '.join(failed_handlers)
        raise declarative.InvalidResponseTypeException(
            "No remaining handlers to handle the response [%s], "
            "used handlers [%s]. [%s]." % (response, handlers_str, self.meta))


class PollsterDefinitionBuilder:

    def __init__(self, definitions):
        self.definitions = definitions

    def build_definitions(self, configurations):
        supported_definitions = []
        for definition in self.definitions:
            if definition.is_field_applicable_to_definition(configurations):
                supported_definitions.append(definition)

        if not supported_definitions:
            raise declarative.DynamicPollsterDefinitionException(
                "Your configurations do not fit any type of DynamicPollsters, "
                "please recheck them. Used configurations are [%s]." %
                configurations)

        definition_name = self.join_supported_definitions_names(
            supported_definitions)

        definition_parents = tuple(supported_definitions)
        definition_attribs = {'extra_definitions': reduce(
            lambda d1, d2: d1 + d2, map(lambda df: df.extra_definitions,
                                        supported_definitions))}
        definition_type = type(definition_name, definition_parents,
                               definition_attribs)
        return definition_type(configurations)

    @staticmethod
    def join_supported_definitions_names(supported_definitions):
        return ''.join(map(lambda df: df.__name__,
                           supported_definitions))


class PollsterSampleExtractor:

    def __init__(self, definitions):
        self.definitions = definitions

    def generate_new_metadata_fields(self, metadata=None,
                                     pollster_definitions=None):
        pollster_definitions =\
            pollster_definitions or self.definitions.configurations
        metadata_mapping = pollster_definitions['metadata_mapping']
        if not metadata_mapping or not metadata:
            return

        metadata_keys = list(metadata.keys())
        for k in metadata_keys:
            if k not in metadata_mapping:
                continue

            new_key = metadata_mapping[k]
            metadata[new_key] = metadata[k]
            LOG.debug("Generating new key [%s] with content [%s] of key [%s]",
                      new_key, metadata[k], k)
            if pollster_definitions['preserve_mapped_metadata']:
                continue

            k_value = metadata.pop(k)
            LOG.debug("Removed key [%s] with value [%s] from "
                      "metadata set that is sent to Gnocchi.", k, k_value)

    def generate_sample(
            self, pollster_sample, pollster_definitions=None, **kwargs):

        pollster_definitions =\
            pollster_definitions or self.definitions.configurations

        metadata = dict()
        if 'metadata_fields' in pollster_definitions:
            for k in pollster_definitions['metadata_fields']:
                val = self.retrieve_attribute_nested_value(
                    pollster_sample, value_attribute=k,
                    definitions=self.definitions.configurations)

                LOG.debug("Assigning value [%s] to metadata key [%s].", val, k)
                metadata[k] = val

        self.generate_new_metadata_fields(
            metadata=metadata, pollster_definitions=pollster_definitions)

        pollster_sample['metadata'] = metadata
        extra_metadata = self.definitions.retrieve_extra_metadata(
            kwargs['manager'], pollster_sample, kwargs['conf'])

        LOG.debug("Extra metadata [%s] collected for sample [%s].",
                  extra_metadata, pollster_sample)
        for key in extra_metadata.keys():
            if key in metadata.keys():
                LOG.warning("The extra metadata key [%s] already exist in "
                            "pollster current metadata set [%s]. Therefore, "
                            "we will ignore it with its value [%s].",
                            key, metadata, extra_metadata[key])
                continue
            metadata[key] = extra_metadata[key]

        return ceilometer_sample.Sample(
            timestamp=ceilometer_utils.isotime(),
            name=pollster_definitions['name'],
            type=pollster_definitions['sample_type'],
            unit=pollster_definitions['unit'],
            volume=pollster_sample['value'],
            user_id=pollster_sample.get("user_id"),
            project_id=pollster_sample.get("project_id"),
            resource_id=pollster_sample.get("id"),
            resource_metadata=metadata)

    def retrieve_attribute_nested_value(self, json_object,
                                        value_attribute=None,
                                        definitions=None, **kwargs):
        if not definitions:
            definitions = self.definitions.configurations

        attribute_key = value_attribute
        if not attribute_key:
            attribute_key = self.definitions.extract_attribute_key()

        LOG.debug(
            "Retrieving the nested keys [%s] from [%s] or pollster [""%s].",
            attribute_key, json_object, definitions["name"])

        keys_and_operations = attribute_key.split("|")
        attribute_key = keys_and_operations[0].strip()

        if attribute_key == ".":
            value = json_object
        else:
            nested_keys = attribute_key.split(".")
            value = reduce(operator.getitem, nested_keys, json_object)

        return self.operate_value(keys_and_operations, value, definitions)

    def operate_value(self, keys_and_operations, value, definitions):
        # We do not have operations to be executed against the value extracted
        if len(keys_and_operations) < 2:
            return value
        for operation in keys_and_operations[1::]:
            # The operation must be performed onto the 'value' variable
            if 'value' not in operation:
                raise declarative.DynamicPollsterDefinitionException(
                    "The attribute field operation [%s] must use the ["
                    "value] variable." % operation, definitions)

            LOG.debug("Executing operation [%s] against value[%s] for "
                      "pollster [%s].", operation, value,
                      definitions["name"])

            value = eval(operation.strip())
            LOG.debug("Result [%s] of operation [%s] for pollster [%s].",
                      value, operation, definitions["name"])
        return value


class SimplePollsterSampleExtractor(PollsterSampleExtractor):

    def generate_single_sample(self, pollster_sample, **kwargs):
        value = self.retrieve_attribute_nested_value(
            pollster_sample)
        value = self.definitions.value_mapper.map_or_skip_value(
            value, pollster_sample)

        if isinstance(value, SkippedSample):
            return value

        pollster_sample['value'] = value

        return self.generate_sample(pollster_sample, **kwargs)

    def extract_sample(self, pollster_sample, **kwargs):
        sample = self.generate_single_sample(pollster_sample, **kwargs)
        if isinstance(sample, SkippedSample):
            return sample
        yield sample


class MultiMetricPollsterSampleExtractor(PollsterSampleExtractor):

    def extract_sample(self, pollster_sample, **kwargs):
        pollster_definitions = self.definitions.configurations

        value = self.retrieve_attribute_nested_value(
            pollster_sample, definitions=pollster_definitions)
        LOG.debug("We are dealing with a multi metric pollster. The "
                  "value we are processing is the following: [%s].",
                  value)

        self.validate_sample_is_list(value)
        sub_metric_placeholder, pollster_name, sub_metric_attribute_name = \
            self.extract_names_attrs()

        value_attribute = \
            self.extract_field_name_from_value_attribute_configuration()
        LOG.debug("Using attribute [%s] to look for values in the "
                  "multi metric pollster [%s] with sample [%s]",
                  value_attribute, pollster_definitions, value)

        pollster_definitions = copy.deepcopy(pollster_definitions)
        yield from self.extract_sub_samples(value, sub_metric_attribute_name,
                                            pollster_name, value_attribute,
                                            sub_metric_placeholder,
                                            pollster_definitions,
                                            pollster_sample, **kwargs)

    def extract_sub_samples(self, value, sub_metric_attribute_name,
                            pollster_name, value_attribute,
                            sub_metric_placeholder, pollster_definitions,
                            pollster_sample, **kwargs):

        for sub_sample in value:
            sub_metric_name = sub_sample[sub_metric_attribute_name]
            new_metric_name = pollster_name.replace(
                sub_metric_placeholder, sub_metric_name)
            pollster_definitions['name'] = new_metric_name

            actual_value = self.retrieve_attribute_nested_value(
                sub_sample, value_attribute, definitions=pollster_definitions)

            pollster_sample['value'] = actual_value

            if self.should_skip_generate_sample(actual_value, sub_sample,
                                                sub_metric_name):
                continue

            yield self.generate_sample(
                pollster_sample, pollster_definitions, **kwargs)

    def extract_field_name_from_value_attribute_configuration(self):
        value_attribute = self.definitions.configurations['value_attribute']
        return self.definitions.pattern_pollster_value_attribute.match(
            value_attribute).group(3)[1::]

    def extract_names_attrs(self):
        pollster_name = self.definitions.configurations['name']
        sub_metric_placeholder = pollster_name.split(".").pop()
        return (sub_metric_placeholder,
                pollster_name,
                self.definitions.pattern_pollster_name.match(
                    "." + sub_metric_placeholder).group(2))

    def validate_sample_is_list(self, value):
        pollster_definitions = self.definitions.configurations
        if not isinstance(value, list):
            raise declarative.DynamicPollsterException(
                "Multi metric pollster defined, but the value [%s]"
                " obtained with [%s] attribute is not a list"
                " of objects."
                % (value,
                   pollster_definitions['value_attribute']),
                pollster_definitions)

    def should_skip_generate_sample(self, actual_value, sub_sample,
                                    sub_metric_name):
        skip_sample_values = \
            self.definitions.configurations['skip_sample_values']
        if actual_value in skip_sample_values:
            LOG.debug(
                "Skipping multi metric sample [%s] because "
                "value [%s] is configured to be skipped in "
                "skip list [%s].", sub_sample, actual_value,
                skip_sample_values)
            return True
        if sub_metric_name in skip_sample_values:
            LOG.debug(
                "Skipping sample [%s] because its sub-metric "
                "name [%s] is configured to be skipped in "
                "skip list [%s].", sub_sample, sub_metric_name,
                skip_sample_values)
            return True
        return False


class PollsterValueMapper:

    def __init__(self, definitions):
        self.definitions = definitions

    def map_or_skip_value(self, value, pollster_sample):
        skip_sample_values = \
            self.definitions.configurations['skip_sample_values']

        if value in skip_sample_values:
            LOG.debug("Skipping sample [%s] because value [%s] "
                      "is configured to be skipped in skip list [%s].",
                      pollster_sample, value, skip_sample_values)
            return SkippedSample()

        return self.execute_value_mapping(value)

    def execute_value_mapping(self, value):
        value_mapping = self.definitions.configurations['value_mapping']
        if not value_mapping:
            return value

        if value in value_mapping:
            old_value = value
            value = value_mapping[value]
            LOG.debug("Value mapped from [%s] to [%s]",
                      old_value, value)
        else:
            default_value = \
                self.definitions.configurations['default_value']
            LOG.warning(
                "Value [%s] was not found in value_mapping [%s]; "
                "therefore, we will use the default [%s].",
                value, value_mapping, default_value)
            value = default_value
        return value


class PollsterDefinition:
    """Represents a dynamic pollster configuration/parameter

    It abstract the job of developers when creating or extending parameters,
    such as validating parameters name, values and so on.
    """

    def __init__(self, name, required=False, on_missing=lambda df: df.default,
                 default=None, validation_regex=None, creatable=True,
                 validator=None):
        """Create a dynamic pollster configuration/parameter

        :param name: the name of the pollster parameter/configuration.
        :param required: indicates if the configuration/parameter is
               optional or not.
        :param on_missing: function that is executed when the
               parameter/configuration is missing.
        :param default: the default value to be used.
        :param validation_regex: the regular expression used to validate the
               name of the configuration/parameter.
        :param creatable: it is an override mechanism to avoid creating
               a configuration/parameter with the default value. The default
               is ``True``; therefore, we always use the default value.
               However, we can disable the use of the default value by
               setting ``False``. When we set this configuration to
               ``False``, the parameter is not added to the definition
               dictionary if not defined by the operator in the pollster
               YAML configuration file.
        :param validator: function used to validate the value of the
               parameter/configuration when it is given by the user. This
               function signature should receive a value that is the value of
               the parameter to be validate.
        """

        self.name = name
        self.required = required
        self.on_missing = on_missing
        self.validation_regex = validation_regex
        self.creatable = creatable
        self.default = default
        if self.validation_regex:
            self.validation_pattern = re.compile(self.validation_regex)
        self.validator = validator

    def validate(self, val):
        if val is None:
            return self.on_missing(self)
        if self.validation_regex and not self.validation_pattern.match(val):
            raise declarative.DynamicPollsterDefinitionException(
                "Pollster %s [%s] does not match [%s]."
                % (self.name, val, self.validation_regex))

        if self.validator:
            self.validator(val)

        return val


class PollsterDefinitions:

    POLLSTER_VALID_NAMES_REGEXP = r"^([\w-]+)(\.[\w-]+)*(\.{[\w-]+})?$"

    EXTERNAL_ENDPOINT_TYPE = "external"

    standard_definitions = [
        PollsterDefinition(name='name', required=True,
                           validation_regex=POLLSTER_VALID_NAMES_REGEXP),
        PollsterDefinition(name='sample_type', required=True,
                           validator=validate_sample_type),
        PollsterDefinition(name='unit', required=True),
        PollsterDefinition(name='endpoint_type', required=True),
        PollsterDefinition(name='url_path', required=True),
        PollsterDefinition(name='metadata_fields', creatable=False),
        PollsterDefinition(name='skip_sample_values', default=[]),
        PollsterDefinition(name='value_mapping', default={}),
        PollsterDefinition(name='default_value', default=-1),
        PollsterDefinition(name='metadata_mapping', default={}),
        PollsterDefinition(name='preserve_mapped_metadata', default=True),
        PollsterDefinition(name='response_entries_key'),
        PollsterDefinition(name='next_sample_url_attribute'),
        PollsterDefinition(name='user_id_attribute', default="user_id"),
        PollsterDefinition(name='resource_id_attribute', default="id"),
        PollsterDefinition(name='project_id_attribute', default="project_id"),
        PollsterDefinition(name='headers'),
        PollsterDefinition(name='timeout', default=30),
        PollsterDefinition(name='extra_metadata_fields_cache_seconds',
                           default=3600),
        PollsterDefinition(name='extra_metadata_fields'),
        PollsterDefinition(name='extra_metadata_fields_skip', default=[{}],
                           validator=validate_extra_metadata_skip_samples),
        PollsterDefinition(name='response_handlers', default=['json'],
                           validator=validate_response_handler),
        PollsterDefinition(name='base_metadata', default={})
    ]

    extra_definitions = []

    def __init__(self, configurations):
        self.configurations = configurations
        self.value_mapper = PollsterValueMapper(self)
        self.definitions = self.map_definitions()
        self.validate_configurations(configurations)
        self.validate_missing()
        self.sample_gatherer = PollsterSampleGatherer(self)
        self.sample_extractor = SimplePollsterSampleExtractor(self)
        self.response_cache = {}

    def validate_configurations(self, configurations):
        for k, v in self.definitions.items():
            if configurations.get(k) is not None:
                self.configurations[k] = self.definitions[k].validate(
                    self.configurations[k])
            elif self.definitions[k].creatable:
                self.configurations[k] = self.definitions[k].default

    @staticmethod
    def is_field_applicable_to_definition(configurations):
        return True

    def map_definitions(self):
        definitions = dict(
            map(lambda df: (df.name, df), self.standard_definitions))
        extra_definitions = dict(
            map(lambda df: (df.name, df), self.extra_definitions))
        definitions.update(extra_definitions)
        return definitions

    def extract_attribute_key(self):
        pass

    def validate_missing(self):
        required_configurations = map(lambda fdf: fdf.name,
                                      filter(lambda df: df.required,
                                             self.definitions.values()))

        missing = list(filter(
            lambda rf: rf not in map(lambda f: f[0],
                                     filter(lambda f: f[1],
                                            self.configurations.items())),
            required_configurations))

        if missing:
            raise declarative.DynamicPollsterDefinitionException(
                "Required fields %s not specified."
                % missing, self.configurations)

    def should_skip_extra_metadata(self, skip, sample):
        match_msg = "Sample [%s] %smatches with configured" \
                    " extra_metadata_fields_skip [%s]."
        if skip == sample:
            LOG.debug(match_msg, sample, "", skip)
            return True
        if not isinstance(skip, dict) or not isinstance(sample, dict):
            LOG.debug(match_msg, sample, "not ", skip)
            return False

        for key in skip:
            if key not in sample:
                LOG.debug(match_msg, sample, "not ", skip)
                return False
            if not self.should_skip_extra_metadata(skip[key], sample[key]):
                LOG.debug(match_msg, sample, "not ", skip)
                return False

        LOG.debug(match_msg, sample, "", skip)
        return True

    def skip_sample(self, request_sample, skips):
        for skip in skips:
            if not skip:
                continue
            if self.should_skip_extra_metadata(skip, request_sample):
                LOG.debug("Skipping extra_metadata_field gathering for "
                          "sample [%s] as defined in the "
                          "extra_metadata_fields_skip [%s]", request_sample,
                          skip)
                return True
        return False

    def retrieve_extra_metadata(self, manager, request_sample, pollster_conf):
        extra_metadata_fields = self.configurations['extra_metadata_fields']
        if extra_metadata_fields:
            extra_metadata_samples = {}
            extra_metadata_by_name = {}
            if not isinstance(extra_metadata_fields, (list, tuple)):
                extra_metadata_fields = [extra_metadata_fields]
            for ext_metadata in extra_metadata_fields:
                ext_metadata.setdefault(
                    'extra_metadata_fields_skip',
                    self.configurations['extra_metadata_fields_skip'])
                ext_metadata.setdefault(
                    'sample_type', self.configurations['sample_type'])
                ext_metadata.setdefault('unit', self.configurations['unit'])
                ext_metadata.setdefault(
                    'value_attribute', ext_metadata.get(
                        'value', self.configurations['value_attribute']))
                ext_metadata['base_metadata'] = {
                    'extra_metadata_captured': extra_metadata_samples,
                    'extra_metadata_by_name': extra_metadata_by_name,
                    'sample': request_sample
                }
                parent_cache_ttl = self.configurations[
                    'extra_metadata_fields_cache_seconds']
                cache_ttl = ext_metadata.get(
                    'extra_metadata_fields_cache_seconds', parent_cache_ttl
                )
                response_cache = self.response_cache
                extra_metadata_pollster = DynamicPollster(
                    ext_metadata, conf=pollster_conf, cache_ttl=cache_ttl,
                    extra_metadata_responses_cache=response_cache,
                )

                skips = ext_metadata['extra_metadata_fields_skip']
                if self.skip_sample(request_sample, skips):
                    continue

                resources = [None]
                if ext_metadata.get('endpoint_type'):
                    resources = manager.discover([
                        extra_metadata_pollster.default_discovery], {})
                samples = extra_metadata_pollster.get_samples(
                    manager, None, resources)
                for sample in samples:
                    self.fill_extra_metadata_samples(
                        extra_metadata_by_name,
                        extra_metadata_samples,
                        sample)
            return extra_metadata_samples

        LOG.debug("No extra metadata to be captured for pollsters [%s] and "
                  "request sample [%s].", self.definitions, request_sample)
        return {}

    def fill_extra_metadata_samples(self, extra_metadata_by_name,
                                    extra_metadata_samples, sample):
        extra_metadata_samples[sample.name] = sample.volume
        LOG.debug("Merging the sample metadata [%s] of the "
                  "extra_metadata_field [%s], with the "
                  "extra_metadata_samples [%s].",
                  sample.resource_metadata,
                  sample.name,
                  extra_metadata_samples)
        for key, value in sample.resource_metadata.items():
            if value is None and key in extra_metadata_samples:
                LOG.debug("Metadata [%s] for extra_metadata_field [%s] "
                          "is None, skipping metadata override by None "
                          "value", key, sample.name)
                continue
            extra_metadata_samples[key] = value
        extra_metadata_by_name[sample.name] = {
            'value': sample.volume,
            'metadata': sample.resource_metadata
        }

        LOG.debug("extra_metadata_samples after merging: [%s].",
                  extra_metadata_samples)


class MultiMetricPollsterDefinitions(PollsterDefinitions):

    MULTI_METRIC_POLLSTER_NAME_REGEXP = r".*(\.{(\w+)})$"
    pattern_pollster_name = re.compile(
        MULTI_METRIC_POLLSTER_NAME_REGEXP)
    MULTI_METRIC_POLLSTER_VALUE_ATTRIBUTE_REGEXP = r"^(\[(\w+)\])((\.\w+)+)$"
    pattern_pollster_value_attribute = re.compile(
        MULTI_METRIC_POLLSTER_VALUE_ATTRIBUTE_REGEXP)

    extra_definitions = [
        PollsterDefinition(
            name='value_attribute', required=True,
            validation_regex=MULTI_METRIC_POLLSTER_VALUE_ATTRIBUTE_REGEXP),
    ]

    def __init__(self, configurations):
        super().__init__(configurations)
        self.sample_extractor = MultiMetricPollsterSampleExtractor(self)

    @staticmethod
    def is_field_applicable_to_definition(configurations):
        return configurations.get(
            'name') and MultiMetricPollsterDefinitions.\
            pattern_pollster_name.match(configurations['name'])

    def extract_attribute_key(self):
        return self.pattern_pollster_value_attribute.match(
            self.configurations['value_attribute']).group(2)


class SingleMetricPollsterDefinitions(PollsterDefinitions):

    extra_definitions = [
        PollsterDefinition(name='value_attribute', required=True)]

    def __init__(self, configurations):
        super().__init__(configurations)

    def extract_attribute_key(self):
        return self.configurations['value_attribute']

    @staticmethod
    def is_field_applicable_to_definition(configurations):
        return not MultiMetricPollsterDefinitions. \
            is_field_applicable_to_definition(configurations)


class PollsterSampleGatherer:

    def __init__(self, definitions):
        self.definitions = definitions
        self.response_handler_chain = ResponseHandlerChain(
            map(VALID_HANDLERS.get,
                self.definitions.configurations['response_handlers']),
            url_path=definitions.configurations['url_path']
        )

    def get_cache_key(self, definitions, **kwargs):
        return self.get_request_linked_samples_url(kwargs, definitions)

    def get_cached_response(self, definitions, **kwargs):
        if self.definitions.cache_ttl == 0:
            return
        cache_key = self.get_cache_key(definitions, **kwargs)
        response_cache = self.definitions.response_cache
        cached_response, max_ttl_for_cache = response_cache.get(
            cache_key, (None, None))

        current_time = time.time()
        if cached_response and max_ttl_for_cache >= current_time:
            LOG.debug("Returning response [%s] for request [%s] as the TTL "
                      "[max=%s, current_time=%s] has not expired yet.",
                      cached_response, definitions,
                      max_ttl_for_cache, current_time)
            return cached_response

        if cached_response and max_ttl_for_cache < current_time:
            LOG.debug("Cleaning cached response [%s] for request [%s] "
                      "as the TTL [max=%s, current_time=%s] has expired.",
                      cached_response, definitions,
                      max_ttl_for_cache, current_time)
            response_cache.pop(cache_key, None)

    def store_cached_response(self, definitions, resp, **kwargs):
        if self.definitions.cache_ttl == 0:
            return
        cache_key = self.get_cache_key(definitions, **kwargs)
        extra_metadata_fields_cache_seconds = self.definitions.cache_ttl
        max_ttl_for_cache = time.time() + extra_metadata_fields_cache_seconds

        cache_tuple = (resp, max_ttl_for_cache)
        self.definitions.response_cache[cache_key] = cache_tuple

    @property
    def default_discovery(self):
        return 'endpoint:' + self.definitions.configurations['endpoint_type']

    def execute_request_get_samples(self, **kwargs):
        return self.execute_request_for_definitions(
            self.definitions.configurations, **kwargs)

    def execute_request_for_definitions(self, definitions, **kwargs):
        if response_dict := self.get_cached_response(definitions, **kwargs):
            url = 'cached'
        else:
            resp, url = self._internal_execute_request_get_samples(
                definitions=definitions, **kwargs)
            response_dict = self.response_handler_chain.handle(resp.text)
            self.store_cached_response(definitions, response_dict, **kwargs)

        entry_size = len(response_dict)
        LOG.debug("Entries [%s] in the DICT for request [%s] "
                  "for dynamic pollster [%s].",
                  response_dict, url, definitions['name'])

        if entry_size > 0:
            samples = self.retrieve_entries_from_response(
                response_dict, definitions)
            url_to_next_sample = self.get_url_to_next_sample(
                response_dict, definitions)

            self.prepare_samples(definitions, samples, **kwargs)

            if url_to_next_sample:
                kwargs['next_sample_url'] = url_to_next_sample
                samples += self.execute_request_for_definitions(
                    definitions=definitions, **kwargs)
            return samples
        return []

    def prepare_samples(
            self, definitions, samples, execute_id_overrides=True, **kwargs):
        if samples and execute_id_overrides:
            for request_sample in samples:
                user_id_attribute = definitions.get(
                    'user_id_attribute', 'user_id')
                project_id_attribute = definitions.get(
                    'project_id_attribute', 'project_id')
                resource_id_attribute = definitions.get(
                    'resource_id_attribute', 'id')

                self.generate_new_attributes_in_sample(
                    request_sample, user_id_attribute, 'user_id')
                self.generate_new_attributes_in_sample(
                    request_sample, project_id_attribute, 'project_id')
                self.generate_new_attributes_in_sample(
                    request_sample, resource_id_attribute, 'id')

    def generate_new_attributes_in_sample(
            self, sample, attribute_key, new_attribute_key):

        if attribute_key == new_attribute_key:
            LOG.debug("We do not need to generate new attribute as the "
                      "attribute_key[%s] and the new_attribute_key[%s] "
                      "configurations are the same.",
                      attribute_key, new_attribute_key)
            return

        if attribute_key:
            attribute_value = self.definitions.sample_extractor.\
                retrieve_attribute_nested_value(sample, attribute_key)

            LOG.debug("Mapped attribute [%s] to value [%s] in sample [%s].",
                      attribute_key, attribute_value, sample)

            sample[new_attribute_key] = attribute_value

    def get_url_to_next_sample(self, resp, definitions):
        linked_sample_extractor = definitions.get('next_sample_url_attribute')

        if not linked_sample_extractor:
            return None

        try:
            return self.definitions.sample_extractor.\
                retrieve_attribute_nested_value(resp, linked_sample_extractor)
        except KeyError:
            LOG.debug("There is no next sample url for the sample [%s] using "
                      "the configuration [%s]", resp, linked_sample_extractor)
        return None

    def _internal_execute_request_get_samples(self, definitions=None,
                                              keystone_client=None, **kwargs):
        if not definitions:
            definitions = self.definitions.configurations

        url = self.get_request_linked_samples_url(kwargs, definitions)
        request_arguments = self.create_request_arguments(definitions)
        LOG.debug("Executing request against [url=%s] with parameters ["
                  "%s] for pollsters [%s]", url, request_arguments,
                  definitions["name"])
        resp = keystone_client.session.get(url, **request_arguments)
        if resp.status_code != requests.codes.ok:
            resp.raise_for_status()
        return resp, url

    def create_request_arguments(self, definitions):
        request_args = {
            "authenticated": True
        }
        request_headers = definitions.get('headers', [])
        if request_headers:
            request_args['headers'] = request_headers
        request_args['timeout'] = definitions.get('timeout', 300)
        return request_args

    def get_request_linked_samples_url(self, kwargs, definitions):
        next_sample_url = kwargs.get('next_sample_url')
        if next_sample_url:
            return self.get_next_page_url(kwargs, next_sample_url)

        LOG.debug("Generating url with [%s] and path [%s].",
                  kwargs, definitions['url_path'])
        return self.get_request_url(
            kwargs, definitions['url_path'])

    def get_next_page_url(self, kwargs, next_sample_url):
        parse_result = urlparse.urlparse(next_sample_url)
        if parse_result.netloc:
            return next_sample_url
        return self.get_request_url(kwargs, next_sample_url)

    def get_request_url(self, kwargs, url_path):
        endpoint = kwargs['resource']
        params = copy.deepcopy(
            self.definitions.configurations.get(
                'base_metadata', {}))
        try:
            url_path = eval(url_path, params)
        except Exception:
            LOG.debug("Cannot eval path [%s] with params [%s],"
                      " using [%s] instead.",
                      url_path, params, url_path)
        return urlparse.urljoin((endpoint
                                 if endpoint.endswith("/")
                                 else (endpoint + "/")),
                                url_path)

    def retrieve_entries_from_response(self, response_json, definitions):
        if isinstance(response_json, list):
            return response_json

        first_entry_name = definitions.get('response_entries_key')

        if not first_entry_name:
            try:
                first_entry_name = next(iter(response_json))
            except RuntimeError as e:
                LOG.debug("Generator threw a StopIteration "
                          "and we need to catch it [%s].", e)
        return self.definitions.sample_extractor.\
            retrieve_attribute_nested_value(response_json, first_entry_name)


class NonOpenStackApisPollsterDefinition(PollsterDefinitions):

    extra_definitions = [
        PollsterDefinition(name='value_attribute', required=True),
        PollsterDefinition(name='module', required=True),
        PollsterDefinition(name='authentication_object', required=True),
        PollsterDefinition(name='barbican_secret_id', default=""),
        PollsterDefinition(name='authentication_parameters', default=""),
        PollsterDefinition(name='endpoint_type')]

    def __init__(self, configurations):
        super().__init__(
            configurations)
        self.sample_gatherer = NonOpenStackApisSamplesGatherer(self)

    @staticmethod
    def is_field_applicable_to_definition(configurations):
        return configurations.get('module')


class HostCommandPollsterDefinition(PollsterDefinitions):

    extra_definitions = [
        PollsterDefinition(name='endpoint_type', required=False),
        PollsterDefinition(name='url_path', required=False),
        PollsterDefinition(name='host_command', required=True)]

    def __init__(self, configurations):
        super().__init__(
            configurations)
        self.sample_gatherer = HostCommandSamplesGatherer(self)

    @staticmethod
    def is_field_applicable_to_definition(configurations):
        return configurations.get('host_command')


class HostCommandSamplesGatherer(PollsterSampleGatherer):

    class Response:
        def __init__(self, text):
            self.text = text

    def get_cache_key(self, definitions, **kwargs):
        return self.get_command(definitions)

    def _internal_execute_request_get_samples(self, definitions, **kwargs):
        command = self.get_command(definitions, **kwargs)
        LOG.debug('Running Host command: [%s]', command)
        result = subprocess.getoutput(command)
        LOG.debug('Host command [%s] result: [%s]', command, result)
        return self.Response(result), command

    def get_command(self, definitions, next_sample_url=None, **kwargs):
        command = next_sample_url or definitions['host_command']
        params = copy.deepcopy(
            self.definitions.configurations.get(
                'base_metadata', {}))
        try:
            command = eval(command, params)
        except Exception:
            LOG.debug("Cannot eval command [%s] with params [%s],"
                      " using [%s] instead.",
                      command, params, command)
        return command

    @property
    def default_discovery(self):
        return 'local_node'


class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):

    @property
    def default_discovery(self):
        return 'barbican:' + \
               self.definitions.configurations['barbican_secret_id']

    def _internal_execute_request_get_samples(self, definitions, **kwargs):
        credentials = kwargs['resource']

        override_credentials = definitions['authentication_parameters']
        if override_credentials:
            credentials = override_credentials

        if not isinstance(credentials, str):
            credentials = self.normalize_credentials_to_string(credentials)

        url = self.get_request_linked_samples_url(kwargs, definitions)

        authenticator_module_name = definitions['module']
        authenticator_class_name = definitions['authentication_object']

        imported_module = __import__(authenticator_module_name)
        authenticator_class = getattr(imported_module,
                                      authenticator_class_name)

        authenticator_arguments = list(map(str.strip, credentials.split(",")))
        authenticator_instance = authenticator_class(*authenticator_arguments)

        request_arguments = self.create_request_arguments(definitions)
        request_arguments["auth"] = authenticator_instance

        LOG.debug("Executing request against [url=%s] with parameters ["
                  "%s] for pollsters [%s]", url, request_arguments,
                  definitions["name"])
        resp = requests.get(url, **request_arguments)

        if resp.status_code != requests.codes.ok:
            raise declarative.NonOpenStackApisDynamicPollsterException(
                "Error while executing request[%s]."
                " Status[%s] and reason [%s]."
                % (url, resp.status_code, resp.reason))

        return resp, url

    @staticmethod
    def normalize_credentials_to_string(credentials):
        if isinstance(credentials, bytes):
            credentials = credentials.decode('utf-8')
        else:
            credentials = str(credentials)
        LOG.debug("Credentials [%s] were not defined as a string. "
                  "Therefore, we converted it to a string like object.",
                  credentials)
        return credentials

    def create_request_arguments(self, definitions):
        request_arguments = super().create_request_arguments(
            definitions)

        request_arguments.pop("authenticated")

        return request_arguments

    def get_request_url(self, kwargs, url_path):
        endpoint = self.definitions.configurations['url_path']
        if endpoint == url_path:
            return url_path
        return urlparse.urljoin((endpoint
                                 if endpoint.endswith("/")
                                 else (endpoint + "/")),
                                url_path)

    def generate_new_attributes_in_sample(
            self, sample, attribute_key, new_attribute_key):
        if attribute_key:
            attribute_value = self.definitions.sample_extractor. \
                retrieve_attribute_nested_value(sample, attribute_key)

            LOG.debug("Mapped attribute [%s] to value [%s] in sample [%s].",
                      attribute_key, attribute_value, sample)

            sample[new_attribute_key] = attribute_value


class SkippedSample:
    pass


class DynamicPollster(plugin_base.PollsterBase):
    # Mandatory name field
    name = ""

    def __init__(self, pollster_definitions={}, conf=None, cache_ttl=0,
                 extra_metadata_responses_cache=None,
                 supported_definitions=[HostCommandPollsterDefinition,
                                        NonOpenStackApisPollsterDefinition,
                                        MultiMetricPollsterDefinitions,
                                        SingleMetricPollsterDefinitions]):
        super().__init__(conf)
        self.supported_definitions = supported_definitions
        LOG.debug("%s instantiated with [%s]", __name__,
                  pollster_definitions)

        self.definitions = PollsterDefinitionBuilder(
            self.supported_definitions).build_definitions(pollster_definitions)
        self.definitions.cache_ttl = cache_ttl
        self.definitions.response_cache = extra_metadata_responses_cache
        if extra_metadata_responses_cache is None:
            self.definitions.response_cache = {}
        self.pollster_definitions = self.definitions.configurations
        if 'metadata_fields' in self.pollster_definitions:
            LOG.debug("Metadata fields configured to [%s].",
                      self.pollster_definitions['metadata_fields'])

        self.name = self.pollster_definitions['name']
        self.obj = self

    @property
    def default_discovery(self):
        return self.definitions.sample_gatherer.default_discovery

    def load_samples(self, resource, manager):
        try:
            return self.definitions.sample_gatherer.\
                execute_request_get_samples(manager=manager,
                                            resource=resource,
                                            keystone_client=manager._keystone)
        except RequestException as e:
            LOG.warning("Error [%s] while loading samples for [%s] "
                        "for dynamic pollster [%s].",
                        e, resource, self.name)

        return list([])

    def get_samples(self, manager, cache, resources):
        if not resources:
            LOG.debug("No resources received for processing.")
            yield None

        for r in resources:
            LOG.debug("Executing get sample for resource [%s].", r)
            samples = self.load_samples(r, manager)
            if not isinstance(samples, (list, tuple)):
                samples = [samples]
            for pollster_sample in samples:
                sample = self.extract_sample(
                    pollster_sample, manager=manager,
                    resource=r, conf=self.conf)
                if isinstance(sample, SkippedSample):
                    continue
                yield from sample

    def extract_sample(self, pollster_sample, **kwargs):
        return self.definitions.sample_extractor.extract_sample(
            pollster_sample, **kwargs)



# Copyright 2014-2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from ceilometer.polling.discovery.endpoint import EndpointDiscovery

from urllib import parse as urlparse

import requests

LOG = log.getLogger(__name__)


class NonOpenStackCredentialsDiscovery(EndpointDiscovery):
    """Barbican secrets discovery

    Discovery that supplies non-OpenStack credentials for the dynamic
    pollster sub-system. This solution uses the EndpointDiscovery to
    find the Barbican URL where we can retrieve the credentials.
    """

    BARBICAN_URL_GET_PAYLOAD_PATTERN = "/v1/secrets/%s/payload"

    def discover(self, manager, param=None):
        barbican_secret = "No secrets found"
        if not param:
            return [barbican_secret]
        barbican_endpoints = super().discover(manager, "key-manager")
        if not barbican_endpoints:
            LOG.warning("No Barbican endpoints found to execute the"
                        " credentials discovery process to [%s].",
                        param)
            return [barbican_secret]
        else:
            LOG.debug("Barbican endpoint found [%s].", barbican_endpoints)

        barbican_server = next(iter(barbican_endpoints))
        barbican_endpoint = self.BARBICAN_URL_GET_PAYLOAD_PATTERN % param
        babrican_url = urlparse.urljoin(barbican_server, barbican_endpoint)

        LOG.debug("Retrieving secrets from: %s.", babrican_url)
        resp = manager._keystone.session.get(babrican_url, authenticated=True)
        if resp.status_code != requests.codes.ok:
            resp.raise_for_status()

        return [resp._content]



# Copyright 2014 eNovance
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import itertools
import socket

from keystoneauth1 import loading
from oslo_config import cfg

import ceilometer.alarm.discovery
import ceilometer.cmd.polling
import ceilometer.compute.discovery
import ceilometer.compute.virt.inspector
import ceilometer.compute.virt.libvirt.utils
import ceilometer.designate_client
import ceilometer.event.converter
import ceilometer.image.discovery
import ceilometer.ipmi.pollsters
import ceilometer.keystone_client
import ceilometer.manila_client
import ceilometer.meter.notifications
import ceilometer.neutron_client
import ceilometer.notification
import ceilometer.nova_client
import ceilometer.objectstore.rgw
import ceilometer.objectstore.swift
import ceilometer.octavia_client
import ceilometer.pipeline.base
import ceilometer.polling.manager
import ceilometer.publisher.messaging
import ceilometer.publisher.utils
import ceilometer.sample
import ceilometer.utils
import ceilometer.volume.discovery


OPTS = [
    cfg.HostAddressOpt('host',
                       default=socket.gethostname(),
                       sample_default='<your_hostname>',
                       help='Hostname, FQDN or IP address of this host. '
                            'Must be valid within AMQP key.'),
    cfg.IntOpt('http_timeout',
               default=600,
               deprecated_for_removal=True,
               deprecated_reason='This option has no effect',
               help='Timeout seconds for HTTP requests. Set it to None to '
                    'disable timeout.'),
    cfg.IntOpt('max_parallel_requests',
               default=64,
               min=1,
               help='Maximum number of parallel requests for '
               'services to handle at the same time.'),
]


def list_opts():
    # FIXME(sileht): re-add pollster namespaces in the generated configfile
    # This have been removed due to a recursive import issue
    return [
        ('DEFAULT',
         itertools.chain(ceilometer.cmd.polling.CLI_OPTS,
                         ceilometer.compute.virt.inspector.OPTS,
                         ceilometer.compute.virt.libvirt.utils.OPTS,
                         ceilometer.objectstore.swift.OPTS,
                         ceilometer.pipeline.base.OPTS,
                         ceilometer.polling.manager.POLLING_OPTS,
                         ceilometer.sample.OPTS,
                         ceilometer.utils.OPTS,
                         OPTS)),
        ('compute', ceilometer.compute.discovery.OPTS),
        ('coordination', [
            cfg.StrOpt(
                'backend_url',
                secret=True,
                help='The backend URL to use for distributed coordination. If '
                'left empty, per-deployment central agent and per-host '
                'compute agent won\'t do workload '
                'partitioning and will only function correctly if a '
                'single instance of that service is running.')
        ]),
        ('event', ceilometer.event.converter.OPTS),
        ('ipmi', ceilometer.ipmi.pollsters.OPTS),
        ('meter', ceilometer.meter.notifications.OPTS),
        ('notification',
         itertools.chain(ceilometer.notification.OPTS,
                         ceilometer.notification.EXCHANGES_OPTS)),
        ('polling', ceilometer.polling.manager.POLLING_OPTS),
        ('publisher', ceilometer.publisher.utils.OPTS),
        ('publisher_notifier', ceilometer.publisher.messaging.NOTIFIER_OPTS),
        ('rgw_admin_credentials', ceilometer.objectstore.rgw.CREDENTIAL_OPTS),
        ('rgw_client', ceilometer.objectstore.rgw.CLIENT_OPTS),
        ('service_types',
         itertools.chain(ceilometer.alarm.discovery.SERVICE_OPTS,
                         ceilometer.designate_client.SERVICE_OPTS,
                         ceilometer.manila_client.SERVICE_OPTS,
                         ceilometer.neutron_client.SERVICE_OPTS,
                         ceilometer.nova_client.SERVICE_OPTS,
                         ceilometer.objectstore.rgw.SERVICE_OPTS,
                         ceilometer.objectstore.swift.SERVICE_OPTS,
                         ceilometer.octavia_client.SERVICE_OPTS,
                         ceilometer.volume.discovery.SERVICE_OPTS,))
    ]


def list_keystoneauth_opts():
    # NOTE(sileht): the configuration file contains only the options
    # for the password plugin that handles keystone v2 and v3 API
    # with discovery. But other options are possible.
    return [('service_credentials', itertools.chain(
        loading.get_session_conf_options(),
        loading.get_auth_common_conf_options(),
        loading.get_auth_plugin_conf_options('password'),
        ceilometer.keystone_client.CLI_OPTS
    ))]



#
# Copyright 2024 cmss, inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for ceilometer/publisher/opentelemetry.py"""

import json
import time
from unittest import mock
import uuid

from oslo_utils import timeutils
from oslotest import base
import requests
from urllib import parse as urlparse

from ceilometer.publisher import opentelemetry_http
from ceilometer import sample
from ceilometer import service


class TestOpentelemetryHttpPublisher(base.BaseTestCase):

    resource_id = str(uuid.uuid4())
    format_time = timeutils.utcnow().isoformat()
    sample_data = [
        sample.Sample(
            name='alpha',
            type=sample.TYPE_CUMULATIVE,
            unit='',
            volume=1,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=format_time,
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='beta',
            type=sample.TYPE_DELTA,
            unit='',
            volume=3,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=format_time,
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='gamma',
            type=sample.TYPE_GAUGE,
            unit='',
            volume=5,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=format_time,
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='delta.epsilon',
            type=sample.TYPE_GAUGE,
            unit='',
            volume=7,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=format_time,
            resource_metadata={'name': 'TestPublish'},
        ),
    ]

    @staticmethod
    def _make_fake_json(sample, format_time):
        struct_time = timeutils.parse_isotime(format_time).timetuple()
        unix_time = int(time.mktime(struct_time))
        if sample.type == "cumulative":
            metric_type = "counter"
        else:
            metric_type = "gauge"
        return {"resource_metrics": [{
            "scope_metrics": [{
                "scope": {
                    "name": "ceilometer",
                    "version": "v1"
                },
                "metrics": [{
                    "name": sample.name.replace(".", "_"),
                    "description": sample.name + " unit:",
                    "unit": "",
                    metric_type: {
                        "data_points": [{
                            "attributes": [{
                                "key": "resource_id",
                                "value": {
                                    "string_value": sample.resource_id
                                }
                            }, {
                                "key": "user_id",
                                "value": {
                                    "string_value": "test"
                                }
                            }, {
                                "key": "project_id",
                                "value": {
                                    "string_value": "test"
                                }
                            }],
                            "start_time_unix_nano": unix_time,
                            "time_unix_nano": unix_time,
                            "as_double": sample.volume,
                            "flags": 0
                        }]}}]}]}]}

    def setUp(self):
        super().setUp()
        self.CONF = service.prepare_service([], [])

    def test_post_samples(self):
        """Test publisher post."""
        parsed_url = urlparse.urlparse(
            'opentelemetryhttp://localhost:4318/v1/metrics')
        publisher = opentelemetry_http.OpentelemetryHttpPublisher(
            self.CONF, parsed_url)

        res = requests.Response()
        res.status_code = 200
        with mock.patch.object(requests.Session, 'post',
                               return_value=res) as m_req:
            publisher.publish_samples(self.sample_data)

        data = []
        for s in self.sample_data:
            data.append(self._make_fake_json(s, self.format_time))
        expected = []
        for d in data:
            expected.append(mock.call('http://localhost:4318/v1/metrics',
                                      auth=None,
                                      cert=None,
                                      data=json.dumps(d),
                                      headers={'Content-type':
                                               'application/json'},
                                      timeout=5,
                                      verify=True))
        self.assertEqual(expected, m_req.mock_calls)

    def test_post_samples_ssl(self):
        """Test publisher post."""
        parsed_url = urlparse.urlparse(
            'opentelemetryhttp://localhost:4318/v1/metrics?ssl=1')
        publisher = opentelemetry_http.OpentelemetryHttpPublisher(
            self.CONF, parsed_url)

        res = requests.Response()
        res.status_code = 200
        with mock.patch.object(requests.Session, 'post',
                               return_value=res) as m_req:
            publisher.publish_samples(self.sample_data)

        data = []
        for s in self.sample_data:
            data.append(self._make_fake_json(s, self.format_time))
        expected = []
        for d in data:
            expected.append(mock.call('https://localhost:4318/v1/metrics',
                                      auth=None,
                                      cert=None,
                                      data=json.dumps(d),
                                      headers={'Content-type':
                                               'application/json'},
                                      timeout=5,
                                      verify=True))
        self.assertEqual(expected, m_req.mock_calls)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Common code for working with volumes
"""
import math

from ceilometer.polling import plugin_base
from ceilometer import sample


class _Base(plugin_base.PollsterBase):
    FIELDS = []

    def extract_metadata(self, obj):
        return {k: getattr(obj, k) for k in self.FIELDS}


class VolumeSizePollster(_Base):
    @property
    def default_discovery(self):
        return 'volumes'

    FIELDS = ['name',
              'status',
              'volume_type',
              'volume_type_id',
              'availability_zone',
              'os-vol-host-attr:host',
              'migration_status',
              'attachments',
              'snapshot_id',
              'source_volid']

    def extract_metadata(self, obj):
        metadata = super().extract_metadata(obj)

        if getattr(obj, "volume_image_metadata", None):
            metadata["image_id"] = obj.volume_image_metadata.get("image_id")
        else:
            metadata["image_id"] = None

        if obj.attachments:
            metadata["instance_id"] = obj.attachments[0]["server_id"]
        else:
            metadata["instance_id"] = None

        return metadata

    def get_samples(self, manager, cache, resources):
        for volume in resources:
            yield sample.Sample(
                name='volume.size',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=volume.size,
                user_id=volume.user_id,
                project_id=getattr(volume,
                                   'os-vol-tenant-attr:tenant_id'),
                resource_id=volume.id,
                resource_metadata=self.extract_metadata(volume),
            )


class VolumeSnapshotSize(_Base):
    @property
    def default_discovery(self):
        return 'volume_snapshots'

    FIELDS = ['name',
              'volume_id',
              'status',
              'description',
              'metadata',
              'os-extended-snapshot-attributes:progress',
              ]

    def get_samples(self, manager, cache, resources):
        for snapshot in resources:
            yield sample.Sample(
                name='volume.snapshot.size',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=snapshot.size,
                user_id=snapshot.user_id,
                project_id=getattr(
                    snapshot,
                    'os-extended-snapshot-attributes:project_id'),
                resource_id=snapshot.id,
                resource_metadata=self.extract_metadata(snapshot),
            )


class VolumeBackupSize(_Base):
    @property
    def default_discovery(self):
        return 'volume_backups'

    FIELDS = ['name',
              'is_incremental',
              'object_count',
              'container',
              'volume_id',
              'status',
              'description']

    def get_samples(self, manager, cache, resources):
        for backup in resources:
            yield sample.Sample(
                name='volume.backup.size',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=backup.size,
                user_id=backup.user_id,
                project_id=getattr(
                    backup, 'os-backup-project-attr:project_id', None),
                resource_id=backup.id,
                resource_metadata=self.extract_metadata(backup),
            )


class _VolumeProviderPoolBase(_Base):
    def extract_metadata(self, obj):
        metadata = super().extract_metadata(obj)
        metadata['pool_name'] = getattr(obj, "pool_name", None)
        return metadata


class VolumeProviderPoolCapacityTotal(_VolumeProviderPoolBase):
    @property
    def default_discovery(self):
        return 'volume_pools'

    def get_samples(self, manager, cache, resources):
        for pool in resources:
            yield sample.Sample(
                name='volume.provider.pool.capacity.total',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=pool.total_capacity_gb,
                user_id=None,
                project_id=None,
                resource_id=pool.name,
                resource_metadata=self.extract_metadata(pool)
            )


class VolumeProviderPoolCapacityFree(_VolumeProviderPoolBase):
    @property
    def default_discovery(self):
        return 'volume_pools'

    def get_samples(self, manager, cache, resources):
        for pool in resources:
            yield sample.Sample(
                name='volume.provider.pool.capacity.free',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=pool.free_capacity_gb,
                user_id=None,
                project_id=None,
                resource_id=pool.name,
                resource_metadata=self.extract_metadata(pool)
            )


class VolumeProviderPoolCapacityProvisioned(_VolumeProviderPoolBase):
    @property
    def default_discovery(self):
        return 'volume_pools'

    def get_samples(self, manager, cache, resources):
        for pool in resources:
            if getattr(pool, 'provisioned_capacity_gb', None):
                yield sample.Sample(
                    name='volume.provider.pool.capacity.provisioned',
                    type=sample.TYPE_GAUGE,
                    unit='GiB',
                    volume=pool.provisioned_capacity_gb,
                    user_id=None,
                    project_id=None,
                    resource_id=pool.name,
                    resource_metadata=self.extract_metadata(pool)
                )


class VolumeProviderPoolCapacityVirtualFree(_VolumeProviderPoolBase):
    @property
    def default_discovery(self):
        return 'volume_pools'

    def get_samples(self, manager, cache, resources):
        for pool in resources:
            if getattr(pool, 'provisioned_capacity_gb', None):
                reserved_size = math.floor(
                    (pool.reserved_percentage / 100) * pool.total_capacity_gb
                )
                max_over_subscription_ratio = 1.0
                if pool.thin_provisioning_support:
                    max_over_subscription_ratio = float(
                        pool.max_over_subscription_ratio
                    )
                value = (
                    max_over_subscription_ratio *
                    (pool.total_capacity_gb - reserved_size) -
                    pool.provisioned_capacity_gb
                )
                yield sample.Sample(
                    name='volume.provider.pool.capacity.virtual_free',
                    type=sample.TYPE_GAUGE,
                    unit='GiB',
                    volume=value,
                    user_id=None,
                    project_id=None,
                    resource_id=pool.name,
                    resource_metadata=self.extract_metadata(pool)
                )


class VolumeProviderPoolCapacityAllocated(_VolumeProviderPoolBase):
    @property
    def default_discovery(self):
        return 'volume_pools'

    def get_samples(self, manager, cache, resources):
        for pool in resources:
            yield sample.Sample(
                name='volume.provider.pool.capacity.allocated',
                type=sample.TYPE_GAUGE,
                unit='GiB',
                volume=pool.allocated_capacity_gb,
                user_id=None,
                project_id=None,
                resource_id=pool.name,
                resource_metadata=self.extract_metadata(pool)
            )



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import fixtures
from oslotest import base

from ceilometer.load_balancer import discovery
from ceilometer.load_balancer import octavia
from ceilometer.polling import manager
from ceilometer.polling import plugin_base
from ceilometer import service


class FakeLoadBalancer:
    """Fake load balancer object mimicking openstacksdk Resource."""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)


class _BaseTestLBPollster(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.addCleanup(mock.patch.stopall)
        self.CONF = service.prepare_service([], [])
        # Mock the openstack.connection.Connection to avoid auth issues
        with mock.patch('openstack.connection.Connection'):
            self.manager = manager.AgentManager(0, self.CONF)
        plugin_base._get_keystone = mock.Mock()
        catalog = (plugin_base._get_keystone.session.auth.get_access.
                   return_value.service_catalog)
        catalog.get_endpoints = mock.MagicMock(
            return_value={'load-balancer': mock.ANY})

    @staticmethod
    def fake_get_loadbalancers():
        return [
            FakeLoadBalancer(
                id='lb-1-uuid',
                name='my-lb-1',
                availability_zone='az-1',
                vip_address='192.168.1.10',
                vip_port_id='port-1-uuid',
                provisioning_status='ACTIVE',
                operating_status='ONLINE',
                provider='amphora',
                flavor_id='flavor-1',
                project_id='tenant-1-uuid',
            ),
            FakeLoadBalancer(
                id='lb-2-uuid',
                name='my-lb-2',
                availability_zone='az-2',
                vip_address='192.168.1.11',
                vip_port_id='port-2-uuid',
                provisioning_status='PENDING_UPDATE',
                operating_status='OFFLINE',
                provider='amphora',
                flavor_id='flavor-1',
                project_id='tenant-2-uuid',
            ),
            FakeLoadBalancer(
                id='lb-3-uuid',
                name='my-lb-3',
                availability_zone=None,
                vip_address='192.168.1.12',
                vip_port_id='port-3-uuid',
                provisioning_status='ERROR',
                operating_status='ERROR',
                provider='amphora',
                flavor_id='flavor-1',
                project_id='tenant-1-uuid',
            ),
            FakeLoadBalancer(
                id='lb-4-uuid',
                name='my-lb-4',
                availability_zone='az-1',
                vip_address='192.168.1.13',
                vip_port_id='port-4-uuid',
                provisioning_status='PENDING_DELETE',
                operating_status='DEGRADED',
                provider='amphora',
                flavor_id='flavor-1',
                project_id='tenant-1-uuid',
            ),
        ]


class TestLoadBalancerOperatingStatusPollster(_BaseTestLBPollster):

    def setUp(self):
        super().setUp()
        self.pollster = octavia.LoadBalancerOperatingStatusPollster(self.CONF)
        fake_lbs = self.fake_get_loadbalancers()
        self.useFixture(fixtures.MockPatch(
            'ceilometer.octavia_client.Client.loadbalancers_list',
            return_value=fake_lbs))

    def test_lb_get_samples(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        self.assertEqual(4, len(samples))
        for field in self.pollster.FIELDS:
            self.assertEqual(
                getattr(self.fake_get_loadbalancers()[0], field),
                samples[0].resource_metadata[field])

    def test_lb_operating_volume(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        # ONLINE = 1, OFFLINE = 3, ERROR = 5, DEGRADED = 4
        self.assertEqual(1, samples[0].volume)
        self.assertEqual(3, samples[1].volume)
        self.assertEqual(5, samples[2].volume)
        self.assertEqual(4, samples[3].volume)

    def test_get_lb_meter_names(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        self.assertEqual({'loadbalancer.operating'},
                         {s.name for s in samples})

    def test_lb_discovery(self):
        with mock.patch('openstack.connection.Connection'):
            discovered_lbs = discovery.LoadBalancerDiscovery(
                self.CONF).discover(self.manager)
        self.assertEqual(4, len(list(discovered_lbs)))


class TestLoadBalancerProvisioningStatusPollster(_BaseTestLBPollster):

    def setUp(self):
        super().setUp()
        self.pollster = octavia.LoadBalancerProvisioningStatusPollster(
            self.CONF)
        fake_lbs = self.fake_get_loadbalancers()
        self.useFixture(fixtures.MockPatch(
            'ceilometer.octavia_client.Client.loadbalancers_list',
            return_value=fake_lbs))

    def test_lb_get_samples(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        self.assertEqual(4, len(samples))
        for field in self.pollster.FIELDS:
            self.assertEqual(
                getattr(self.fake_get_loadbalancers()[0], field),
                samples[0].resource_metadata[field])

    def test_lb_provisioning_volume(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        # ACTIVE = 1, PENDING_UPDATE = 5, ERROR = 3, PENDING_DELETE = 6
        self.assertEqual(1, samples[0].volume)
        self.assertEqual(5, samples[1].volume)
        self.assertEqual(3, samples[2].volume)
        self.assertEqual(6, samples[3].volume)

    def test_get_lb_meter_names(self):
        samples = list(self.pollster.get_samples(
            self.manager, {},
            resources=self.fake_get_loadbalancers()))
        self.assertEqual({'loadbalancer.provisioning'},
                         {s.name for s in samples})


class TestLoadBalancerPollsterUnknownStatus(_BaseTestLBPollster):

    def test_unknown_operating_status(self):
        pollster = octavia.LoadBalancerOperatingStatusPollster(self.CONF)
        fake_lb = FakeLoadBalancer(
            id='lb-unknown-uuid',
            name='my-lb-unknown',
            availability_zone=None,
            vip_address='192.168.1.99',
            vip_port_id='port-unknown-uuid',
            provisioning_status='ACTIVE',
            operating_status='UNKNOWN_STATUS',
            provider='amphora',
            flavor_id='flavor-1',
            project_id='tenant-1-uuid',
        )
        samples = list(pollster.get_samples(
            self.manager, {}, resources=[fake_lb]))
        self.assertEqual(1, len(samples))
        self.assertEqual(-1, samples[0].volume)

    def test_unknown_provisioning_status(self):
        pollster = octavia.LoadBalancerProvisioningStatusPollster(self.CONF)
        fake_lb = FakeLoadBalancer(
            id='lb-unknown-uuid',
            name='my-lb-unknown',
            availability_zone=None,
            vip_address='192.168.1.99',
            vip_port_id='port-unknown-uuid',
            provisioning_status='UNKNOWN_STATUS',
            operating_status='ONLINE',
            provider='amphora',
            flavor_id='flavor-1',
            project_id='tenant-1-uuid',
        )
        samples = list(pollster.get_samples(
            self.manager, {}, resources=[fake_lb]))
        self.assertEqual(1, len(samples))
        self.assertEqual(-1, samples[0].volume)



#
# Copyright 2016 IBM
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for ceilometer/publisher/prometheus.py"""

import datetime
from unittest import mock
import uuid

from oslo_utils import timeutils
from oslotest import base
import requests
from urllib import parse as urlparse

from ceilometer.publisher import prometheus
from ceilometer import sample
from ceilometer import service


class TestPrometheusPublisher(base.BaseTestCase):

    resource_id = str(uuid.uuid4())

    sample_data = [
        sample.Sample(
            name='alpha',
            type=sample.TYPE_CUMULATIVE,
            unit='',
            volume=1,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=timeutils.utcnow().isoformat(),
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='beta',
            type=sample.TYPE_DELTA,
            unit='',
            volume=3,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=timeutils.utcnow().isoformat(),
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='gamma',
            type=sample.TYPE_GAUGE,
            unit='',
            volume=5,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=datetime.datetime.now().isoformat(),
            resource_metadata={'name': 'TestPublish'},
        ),
        sample.Sample(
            name='delta.epsilon',
            type=sample.TYPE_GAUGE,
            unit='',
            volume=7,
            user_id='test',
            project_id='test',
            resource_id=resource_id,
            timestamp=datetime.datetime.now().isoformat(),
            resource_metadata={'name': 'TestPublish'},
        ),
    ]

    def setUp(self):
        super().setUp()
        self.CONF = service.prepare_service([], [])

    def test_post_samples(self):
        """Test publisher post."""
        parsed_url = urlparse.urlparse(
            'prometheus://localhost:90/metrics/job/os')
        publisher = prometheus.PrometheusPublisher(self.CONF, parsed_url)

        res = requests.Response()
        res.status_code = 200
        with mock.patch.object(requests.Session, 'post',
                               return_value=res) as m_req:
            publisher.publish_samples(self.sample_data)

        data = """# TYPE alpha counter
alpha{{resource_id="{}", user_id="test", project_id="test"}} 1
beta{{resource_id="{}", user_id="test", project_id="test"}} 3
# TYPE gamma gauge
gamma{{resource_id="{}", user_id="test", project_id="test"}} 5
# TYPE delta_epsilon gauge
delta_epsilon{{resource_id="{}", user_id="test", project_id="test"}} 7
""".format(self.resource_id, self.resource_id, self.resource_id,
           self.resource_id)

        expected = [
            mock.call('http://localhost:90/metrics/job/os',
                      auth=None,
                      cert=None,
                      data=data,
                      headers={'Content-type': 'plain/text'},
                      timeout=5,
                      verify=True)
        ]
        self.assertEqual(expected, m_req.mock_calls)

    def test_post_samples_ssl(self):
        """Test publisher post."""
        parsed_url = urlparse.urlparse(
            'prometheus://localhost:90/metrics/job/os?ssl=1')
        publisher = prometheus.PrometheusPublisher(self.CONF, parsed_url)

        res = requests.Response()
        res.status_code = 200
        with mock.patch.object(requests.Session, 'post',
                               return_value=res) as m_req:
            publisher.publish_samples(self.sample_data)

        data = """# TYPE alpha counter
alpha{{resource_id="{}", user_id="test", project_id="test"}} 1
beta{{resource_id="{}", user_id="test", project_id="test"}} 3
# TYPE gamma gauge
gamma{{resource_id="{}", user_id="test", project_id="test"}} 5
# TYPE delta_epsilon gauge
delta_epsilon{{resource_id="{}", user_id="test", project_id="test"}} 7
""".format(self.resource_id, self.resource_id, self.resource_id,
           self.resource_id)

        expected = [
            mock.call('https://localhost:90/metrics/job/os',
                      auth=None,
                      cert=None,
                      data=data,
                      headers={'Content-type': 'plain/text'},
                      timeout=5,
                      verify=True)
        ]
        self.assertEqual(expected, m_req.mock_calls)



#
# Copyright 2012 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ceilometer.alarm import aodh
from ceilometer.polling import manager
from ceilometer import service
import ceilometer.tests.base as base

ALARM_METRIC_LIST = [
    {
        'evaluation_results': [{
            'alarm_id': 'b8e17f58-089a-43fc-a96b-e9bcac4d4b53',
            'project_id': '2dd8edd6c8c24f49bf04670534f6b357',
            'state_counters': {
                'ok': 2,
                'alarm': 5,
                'insufficient data': 0,
            }
        }, {
            'alarm_id': 'fa386719-67e3-42ff-aec8-17e547dac77a',
            'project_id': 'd45b070bcce04ca99546128a40854e7c',
            'state_counters': {
                'ok': 50,
                'alarm': 3,
                'insufficient data': 10,
            }
        }],
    },
]


class TestAlarmEvaluationResultPollster(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        conf = service.prepare_service([], [])
        self.manager = manager.AgentManager(0, conf)
        self.pollster = aodh.EvaluationResultPollster(conf)

    def test_alarm_pollster(self):
        alarm_samples = list(
            self.pollster.get_samples(self.manager,
                                      {},
                                      resources=ALARM_METRIC_LIST))
        self.assertEqual(6, len(alarm_samples))
        self.assertEqual('alarm.evaluation_result', alarm_samples[0].name)
        self.assertEqual(2, alarm_samples[0].volume)
        self.assertEqual('2dd8edd6c8c24f49bf04670534f6b357',
                         alarm_samples[0].project_id)
        self.assertEqual('b8e17f58-089a-43fc-a96b-e9bcac4d4b53',
                         alarm_samples[0].resource_id)
        self.assertEqual('ok',
                         alarm_samples[0].resource_metadata['alarm_state'])



# Copyright 2016 Sungard Availability Services
# Copyright 2016 Red Hat
# Copyright 2012 eNovance <licensing@enovance.com>
# Copyright 2013 IBM Corp
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from ceilometer.network.services import base
from ceilometer import sample

LOG = log.getLogger(__name__)


class FloatingIPPollster(base.BaseServicesPollster):

    FIELDS = ['router_id',
              'status',
              'floating_network_id',
              'fixed_ip_address',
              'port_id',
              'floating_ip_address',
              ]

    @property
    def default_discovery(self):
        return 'fip_services'

    def get_samples(self, manager, cache, resources):

        for fip in resources or []:
            LOG.debug("FLOATING IP : %s", fip)
            status = self.get_status_id(fip['status'])
            if status == -1:
                LOG.warning(
                    "Unknown status %(status)s for floating IP address "
                    "%(address)s (%(id)s), setting volume to -1",
                    {"status": fip['status'],
                     "address": fip['floating_ip_address'],
                     "id": fip['id']})
            yield sample.Sample(
                name='ip.floating',
                type=sample.TYPE_GAUGE,
                unit='ip',
                volume=status,
                user_id=fip.get('user_id'),
                project_id=fip['tenant_id'],
                resource_id=fip['id'],
                resource_metadata=self.extract_metadata(fip)
            )



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ceilometer.polling import manager
from ceilometer import service
from ceilometer.tests import base


class PollingTestCase(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.CONF = service.prepare_service([], [])
        self.poll_cfg = {'sources': [{'name': 'test_source',
                                      'interval': 600,
                                      'meters': ['a']}]}

    def _build_and_set_new_polling(self):
        name = self.cfg2file(self.poll_cfg)
        self.CONF.set_override('cfg_file', name, group='polling')

    def test_no_name(self):
        del self.poll_cfg['sources'][0]['name']
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_no_interval(self):
        del self.poll_cfg['sources'][0]['interval']
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_invalid_string_interval(self):
        self.poll_cfg['sources'][0]['interval'] = 'string'
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_get_interval(self):
        self._build_and_set_new_polling()
        poll_manager = manager.PollingManager(self.CONF)
        source = poll_manager.sources[0]
        self.assertEqual(600, source.get_interval())

    def test_invalid_resources(self):
        self.poll_cfg['sources'][0]['resources'] = {'invalid': 1}
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_resources(self):
        resources = ['test1://', 'test2://']
        self.poll_cfg['sources'][0]['resources'] = resources
        self._build_and_set_new_polling()
        poll_manager = manager.PollingManager(self.CONF)
        self.assertEqual(resources, poll_manager.sources[0].resources)

    def test_no_resources(self):
        self._build_and_set_new_polling()
        poll_manager = manager.PollingManager(self.CONF)
        self.assertEqual(0, len(poll_manager.sources[0].resources))

    def test_check_meters_include_exclude_same(self):
        self.poll_cfg['sources'][0]['meters'] = ['a', '!a']
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_check_meters_include_exclude(self):
        self.poll_cfg['sources'][0]['meters'] = ['a', '!b']
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)

    def test_check_meters_wildcard_included(self):
        self.poll_cfg['sources'][0]['meters'] = ['a', '*']
        self._build_and_set_new_polling()
        self.assertRaises(manager.PollingException,
                          manager.PollingManager, self.CONF)



#
# Copyright 2014 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from ceilometer.pipeline import base
from ceilometer.pipeline import sample as pipeline
from ceilometer import sample
from ceilometer.tests.unit import pipeline_base


class TestDecoupledPipeline(pipeline_base.BasePipelineTestCase):
    def _setup_pipeline_cfg(self):
        source = {'name': 'test_source',
                  'meters': ['a'],
                  'sinks': ['test_sink']}
        sink = {'name': 'test_sink',
                'publishers': ['test://']}
        self.pipeline_cfg = {'sources': [source], 'sinks': [sink]}

    def _augment_pipeline_cfg(self):
        self.pipeline_cfg['sources'].append({
            'name': 'second_source',
            'meters': ['b'],
            'sinks': ['second_sink']
        })
        self.pipeline_cfg['sinks'].append({
            'name': 'second_sink',
            'publishers': ['new'],
        })

    def _break_pipeline_cfg(self):
        self.pipeline_cfg['sources'].append({
            'name': 'second_source',
            'meters': ['b'],
            'sinks': ['second_sink']
        })
        self.pipeline_cfg['sinks'].append({
            'name': 'second_sink',
            'publishers': ['except'],
        })

    def _dup_pipeline_name_cfg(self):
        self.pipeline_cfg['sources'].append({
            'name': 'test_source',
            'meters': ['b'],
            'sinks': ['test_sink']
        })

    def _set_pipeline_cfg(self, field, value):
        if field in self.pipeline_cfg['sources'][0]:
            self.pipeline_cfg['sources'][0][field] = value
        else:
            self.pipeline_cfg['sinks'][0][field] = value

    def _extend_pipeline_cfg(self, field, value):
        if field in self.pipeline_cfg['sources'][0]:
            self.pipeline_cfg['sources'][0][field].extend(value)
        else:
            self.pipeline_cfg['sinks'][0][field].extend(value)

    def _unset_pipeline_cfg(self, field):
        if field in self.pipeline_cfg['sources'][0]:
            del self.pipeline_cfg['sources'][0][field]
        else:
            del self.pipeline_cfg['sinks'][0][field]

    def test_source_no_sink(self):
        del self.pipeline_cfg['sinks']
        self._exception_create_pipelinemanager()

    def test_source_dangling_sink(self):
        self.pipeline_cfg['sources'].append({
            'name': 'second_source',
            'meters': ['b'],
            'sinks': ['second_sink']
        })
        self._exception_create_pipelinemanager()

    def test_sink_no_source(self):
        del self.pipeline_cfg['sources']
        self._exception_create_pipelinemanager()

    def test_source_with_multiple_sinks(self):
        meter_cfg = ['a', 'b']
        self._set_pipeline_cfg('meters', meter_cfg)
        self.pipeline_cfg['sinks'].append({
            'name': 'second_sink',
            'publishers': ['new'],
        })
        self.pipeline_cfg['sources'][0]['sinks'].append('second_sink')
        self._build_and_set_new_pipeline()
        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
        with pipeline_manager.publisher() as p:
            p([self.test_counter])

        self.test_counter = sample.Sample(
            name='b',
            type=self.test_counter.type,
            volume=self.test_counter.volume,
            unit=self.test_counter.unit,
            user_id=self.test_counter.user_id,
            project_id=self.test_counter.project_id,
            resource_id=self.test_counter.resource_id,
            timestamp=self.test_counter.timestamp,
            resource_metadata=self.test_counter.resource_metadata,
        )

        with pipeline_manager.publisher() as p:
            p([self.test_counter])

        self.assertEqual(2, len(pipeline_manager.pipelines))
        self.assertEqual('test_source:test_sink',
                         str(pipeline_manager.pipelines[0]))
        self.assertEqual('test_source:second_sink',
                         str(pipeline_manager.pipelines[1]))
        test_publisher = pipeline_manager.pipelines[0].publishers[0]
        new_publisher = pipeline_manager.pipelines[1].publishers[0]
        for publisher in (test_publisher, new_publisher):
            self.assertEqual(2, len(publisher.samples))
            self.assertEqual(2, publisher.calls)
            self.assertEqual('a', getattr(publisher.samples[0], "name"))
            self.assertEqual('b', getattr(publisher.samples[1], "name"))

    def test_multiple_sources_with_single_sink(self):
        self.pipeline_cfg['sources'].append({
            'name': 'second_source',
            'meters': ['b'],
            'sinks': ['test_sink']
        })
        self._build_and_set_new_pipeline()
        pipeline_manager = pipeline.SamplePipelineManager(self.CONF)
        with pipeline_manager.publisher() as p:
            p([self.test_counter])

        self.test_counter = sample.Sample(
            name='b',
            type=self.test_counter.type,
            volume=self.test_counter.volume,
            unit=self.test_counter.unit,
            user_id=self.test_counter.user_id,
            project_id=self.test_counter.project_id,
            resource_id=self.test_counter.resource_id,
            timestamp=self.test_counter.timestamp,
            resource_metadata=self.test_counter.resource_metadata,
        )

        with pipeline_manager.publisher() as p:
            p([self.test_counter])

        self.assertEqual(2, len(pipeline_manager.pipelines))
        self.assertEqual('test_source:test_sink',
                         str(pipeline_manager.pipelines[0]))
        self.assertEqual('second_source:test_sink',
                         str(pipeline_manager.pipelines[1]))
        test_publisher = pipeline_manager.pipelines[0].publishers[0]
        another_publisher = pipeline_manager.pipelines[1].publishers[0]
        for publisher in [test_publisher, another_publisher]:
            self.assertEqual(2, len(publisher.samples))
            self.assertEqual(2, publisher.calls)
            self.assertEqual('a', getattr(publisher.samples[0], "name"))
            self.assertEqual('b', getattr(publisher.samples[1], "name"))

    def test_duplicated_sinks_names(self):
        self.pipeline_cfg['sinks'].append({
            'name': 'test_sink',
            'publishers': ['except'],
        })
        self._build_and_set_new_pipeline()
        self.assertRaises(base.PipelineException,
                          pipeline.SamplePipelineManager, self.CONF)

    def test_duplicated_source_names(self):
        self.pipeline_cfg['sources'].append({
            'name': 'test_source',
            'meters': ['a'],
            'sinks': ['test_sink']
        })
        self._build_and_set_new_pipeline()
        self.assertRaises(base.PipelineException,
                          pipeline.SamplePipelineManager, self.CONF)
